from abaqus.CustomKernel.CommandRegister import CommandRegister
from abaqus.CustomKernel.RegisteredDictionary import RegisteredDictionary
from abaqus.CustomKernel.RegisteredList import RegisteredList
from abaqus.CustomKernel.RegisteredTuple import RegisteredTuple
from abaqus.CustomKernel.RepositorySupport import RepositorySupport



import typing

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc


@abaqus_class_doc
class Transform:
    """The MakeSketchTransform method creates a Transform object. The Transform object has no
    direct constructor. A :py:class:`~abaqus.BasicGeometry.Transform.Transform` object is a 4×3 matrix of Floats that represents the
    transformation from sketch coordinates to assembly coordinates or to part coordinates.

    .. note:: 
        This object can be accessed by::

            import part
            import assembly
    """

    @abaqus_method_doc
    def matrix(self):
        """This method returns the transformation matrix as a tuple of 12 Floats.

        Returns
        -------
        typing.Tuple[float, float, float, float, float, float, float, float, float, float, float, float]
            A tuple of 12 Floats.

        """
        ...



from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from .ConnectorBehaviorOption import ConnectorBehaviorOption
from .ConnectorOptions import ConnectorOptions
from .ConnectorPotentialArray import ConnectorPotentialArray
from ..UtilityAndView.abaqusConstants import *


@abaqus_class_doc
class ConnectorPlasticity(ConnectorBehaviorOption):
    """The ConnectorPlasticity object defines Plastic behavior for one or more components of a
    connector's relative motion.
    The ConnectorPlasticity object is derived from the ConnectorBehaviorOption object.

    .. note:: 
        This object can be accessed by::

            import section
            mdb.models[name].sections[name].behaviorOptions[i]
            import odbSection
            session.odbs[name].sections[name].behaviorOptions[i]

        The table data for this object are:
        
        Table data for **isotropicTable**:
        
        - If **isotropicType** = TABULAR, then each sequence of the table data specifies the following:
        
            - Equivalent yield force or moment defining the size of the elastic range.
            - Equivalent relative Plastic motion.
            - Equivalent relative Plastic motion rate.
            - Temperature, if the data depend on temperature.
            - Value of the first field variable, if the data depend on field variables.
            - Value of the second field variable.
            - Etc.
        - If **isotropicType** = EXPONENTIAL_LAW, then each sequence of the table data specifies the following:
            
            - Equivalent force or moment defining the size of the elastic range at zero Plastic motion.
            - Isotropic hardening parameter QinfQinf.
            - Isotropic hardening parameter bb.
            - Temperature, if the data depend on temperature.
            - Value of the first field variable, if the data depend on field variables.
            - Value of the second field variable.
            - Etc.
            
        Table data for **kinematicTable**:
        
        - If **kinematicType** = HALF_CYCLE, then each sequence of the table data specifies the following:
            
            - Yield force or moment.
            - Connector relative Plastic motion.
            - Temperature, if the data depend on temperature.
            - Value of the first field variable, if the data depend on field variables.
            - Value of the second field variable.
            - Etc.
        - If **kinematicType** = STABILIZED, then each sequence of the table data specifies the following:
            
            - Yield force or moment.
            - Connector relative Plastic motion.
            - Connector relative constitutive motion range.
            - Temperature, if the data depend on temperature.
            - Value of the first field variable, if the data depend on field variables.
            - Value of the second field variable.
            - Etc.
        - If **kinematicType** = PARAMETERS, then each sequence of the table data specifies the following:
            
            - Yield force or moment at zero relative Plastic motion.
            - Kinematic hardening parameter CC.
            - Kinematic hardening parameter γγ. Set γγ=0 to specify linear Ziegler kinematic hardening.
            - Temperature, if the data depend on temperature.
            - Value of the first field variable, if the data depend on field variables.
            - Value of the second field variable.
            - Etc.

        The corresponding analysis keywords are:

        - CONNECTOR PLASTICITY
        - CONNECTOR HARDENING
        - CONNECTOR POTENTIAL
    """

    #: A :py:class:`~abaqus.Connector.ConnectorOptions.ConnectorOptions` object specifying the ConnectorOptions used to define tabular options
    #: for the isotropic hardening table.
    isotropicOptions: ConnectorOptions = ConnectorOptions()

    #: A :py:class:`~abaqus.Connector.ConnectorOptions.ConnectorOptions` object specifying the ConnectorOptions used to define tabular options
    #: for the kinematic hardening table.
    kinematicOptions: ConnectorOptions = ConnectorOptions()

    #: A SymbolicConstant specifying whether or not the behavior is coupled. Possible values
    #: are UNCOUPLED and COUPLED. The default value is UNCOUPLED.
    coupling: SymbolicConstant = UNCOUPLED

    #: A Boolean specifying whether isotropic hardening data will be used. The default value is
    #: ON.If **isotropic** = OFF, then **kinematic** must be specified as ON.
    isotropic: Boolean = ON

    #: A SymbolicConstant specifying the type of isotropic hardening to be specified. Possible
    #: values are TABULAR and EXPONENTIAL_LAW. The default value is TABULAR.This argument is
    #: applicable only if **isotropic** = ON.
    isotropicType: SymbolicConstant = TABULAR

    #: A Boolean specifying whether the isotropic data depend on temperature. The default value
    #: is OFF.This argument is applicable only if **isotropic** = ON.
    isotropicTemperature: Boolean = OFF

    #: An Int specifying the number of field variable dependencies for the isotropic data. The
    #: default value is 0.This argument is applicable only if **isotropic** = ON.
    isotropicDependencies: int = 0

    #: A Boolean specifying whether kinematic hardening data will be used. The default value is
    #: OFF.If **kinematic** = OFF, then **isotropic** must be specified as ON.
    kinematic: Boolean = OFF

    #: A SymbolicConstant specifying the type of kinematic hardening to be specified. Possible
    #: values are HALF_CYCLE, STABILIZED, and PARAMETERS. The default value is HALF_CYCLE.This
    #: argument is applicable only if **kinematic** = ON.
    kinematicType: SymbolicConstant = HALF_CYCLE

    #: A Boolean specifying whether the kinematic data depend on temperature. The default value
    #: is OFF.This argument is applicable only if **kinematic** = ON.
    kinematicTemperature: Boolean = OFF

    #: An Int specifying the number of field variable dependencies for the kinematic data. The
    #: default value is 0.This argument is applicable only if **kinematic** = ON.
    kinematicDependencies: int = 0

    #: A SymbolicConstant specifying the contribution operator for the force potential
    #: contributions. Possible values are SUM and MAXIMUM. The default value is SUM.This
    #: argument is applicable only if **coupling** = COUPLED.
    forcePotentialOperator: SymbolicConstant = SUM

    #: A Float specifying the number equal to the inverse of the overall exponent in the force
    #: potential definition. The default value is 2.0.This argument is applicable only if
    #: **coupling** = COUPLED and if **forcePotentialOperator** = SUM.
    forcePotentialExponent: float = 2

    #: A :py:class:`~abaqus.Connector.ConnectorPotentialArray.ConnectorPotentialArray` object specifying one ConnectorPotential object for each force
    #: potential contribution. This member can be specified only if **coupling** = COUPLED.
    connectorPotentials: ConnectorPotentialArray = None

    #: A sequence of sequences of Floats specifying isotropic plasticity properties. Items in
    #: the **isotropicTable** data are described below. This argument is applicable only if
    #: **isotropic** = ON. The default value is an empty sequence.
    isotropicTable: tuple = ()

    #: A sequence of sequences of Floats specifying kinematic plasticity properties. Items in
    #: the **kinematicTable** data are described below. This argument is applicable only if
    #: **kinematic** = ON. The default value is an empty sequence.
    kinematicTable: tuple = ()

    #: A sequence of Ints specifying the components of relative motion for which the behavior
    #: is defined. Possible values are 1 ≤ **components** ≤ 6. Only available components can be
    #: specified. This argument can be specified only if **coupling** = UNCOUPLED. The default
    #: value is an empty sequence.
    components: tuple = ()

    @abaqus_method_doc
    def __init__(
        self,
        coupling: SymbolicConstant = UNCOUPLED,
        isotropic: Boolean = ON,
        isotropicType: SymbolicConstant = TABULAR,
        isotropicTemperature: Boolean = OFF,
        isotropicDependencies: int = 0,
        kinematic: Boolean = OFF,
        kinematicType: SymbolicConstant = HALF_CYCLE,
        kinematicTemperature: Boolean = OFF,
        kinematicDependencies: int = 0,
        forcePotentialOperator: SymbolicConstant = SUM,
        forcePotentialExponent: float = 2,
        connectorPotentials: ConnectorPotentialArray = None,
        isotropicTable: tuple = (),
        kinematicTable: tuple = (),
        components: tuple = (),
    ):
        """This method creates a connector plasticity behavior option for a ConnectorSection
        object.

        .. note:: 
            This function can be accessed by::

                import connectorBehavior
                connectorBehavior.ConnectorPlasticity
                import odbConnectorBehavior
                odbConnectorBehavior.ConnectorPlasticity

        Parameters
        ----------
        coupling
            A SymbolicConstant specifying whether or not the behavior is coupled. Possible values
            are UNCOUPLED and COUPLED. The default value is UNCOUPLED.
        isotropic
            A Boolean specifying whether isotropic hardening data will be used. The default value is
            ON.If **isotropic** = OFF, then **kinematic** must be specified as ON.
        isotropicType
            A SymbolicConstant specifying the type of isotropic hardening to be specified. Possible
            values are TABULAR and EXPONENTIAL_LAW. The default value is TABULAR.This argument is
            applicable only if **isotropic** = ON.
        isotropicTemperature
            A Boolean specifying whether the isotropic data depend on temperature. The default value
            is OFF.This argument is applicable only if **isotropic** = ON.
        isotropicDependencies
            An Int specifying the number of field variable dependencies for the isotropic data. The
            default value is 0.This argument is applicable only if **isotropic** = ON.
        kinematic
            A Boolean specifying whether kinematic hardening data will be used. The default value is
            OFF.If **kinematic** = OFF, then **isotropic** must be specified as ON.
        kinematicType
            A SymbolicConstant specifying the type of kinematic hardening to be specified. Possible
            values are HALF_CYCLE, STABILIZED, and PARAMETERS. The default value is HALF_CYCLE.This
            argument is applicable only if **kinematic** = ON.
        kinematicTemperature
            A Boolean specifying whether the kinematic data depend on temperature. The default value
            is OFF.This argument is applicable only if **kinematic** = ON.
        kinematicDependencies
            An Int specifying the number of field variable dependencies for the kinematic data. The
            default value is 0.This argument is applicable only if **kinematic** = ON.
        forcePotentialOperator
            A SymbolicConstant specifying the contribution operator for the force potential
            contributions. Possible values are SUM and MAXIMUM. The default value is SUM.This
            argument is applicable only if **coupling** = COUPLED.
        forcePotentialExponent
            A Float specifying the number equal to the inverse of the overall exponent in the force
            potential definition. The default value is 2.0.This argument is applicable only if
            **coupling** = COUPLED and if **forcePotentialOperator** = SUM.
        connectorPotentials
            A :py:class:`~abaqus.Connector.ConnectorPotentialArray.ConnectorPotentialArray` object specifying one ConnectorPotential object for each force
            potential contribution. This member can be specified only if **coupling** = COUPLED.
        isotropicTable
            A sequence of sequences of Floats specifying isotropic plasticity properties. Items in
            the **isotropicTable** data are described below. This argument is applicable only if
            **isotropic** = ON. The default value is an empty sequence.
        kinematicTable
            A sequence of sequences of Floats specifying kinematic plasticity properties. Items in
            the **kinematicTable** data are described below. This argument is applicable only if
            **kinematic** = ON. The default value is an empty sequence.
        components
            A sequence of Ints specifying the components of relative motion for which the behavior
            is defined. Possible values are 1 ≤ **components** ≤ 6. Only available components can be
            specified. This argument can be specified only if **coupling** = UNCOUPLED. The default
            value is an empty sequence.

        Returns
        -------
        ConnectorPlasticity
            A :py:class:`~abaqus.Connector.ConnectorPlasticity.ConnectorPlasticity` object.

        Raises
        ------
        ValueError and TextError
        """
        super().__init__()

    @abaqus_method_doc
    def setValues(self, *args, **kwargs):
        """This method modifies the ConnectorPlasticity object.

        Raises
        ------
        ValueError
        """
        ...



import typing

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from .DesignResponse import DesignResponse
from .GeometricRestriction import GeometricRestriction
from .ObjectiveFunction import ObjectiveFunction
from .OptimizationConstraint import OptimizationConstraint
from .OptimizationTask import OptimizationTask
from .StopCondition import StopCondition
from ..UtilityAndView.abaqusConstants import *


@abaqus_class_doc
class ShapeTask(OptimizationTask):
    """The ShapeTask object defines a shape task.
    The ShapeTask object is derived from the OptimizationTask object.

    .. note:: 
        This object can be accessed by::

            import optimization
            mdb.models[name].optimizationTasks[name]
    """

    #: A repository of DesignResponse objects.
    designResponses: typing.Dict[str, DesignResponse] = {}

    #: A repository of ObjectiveFunction objects.
    objectiveFunctions: typing.Dict[str, ObjectiveFunction] = {}

    #: A repository of OptimizationConstraint objects.
    optimizationConstraints: typing.Dict[str, OptimizationConstraint] = {}

    #: A repository of GeometricRestriction objects.
    geometricRestrictions: typing.Dict[str, GeometricRestriction] = {}

    #: A repository of StopCondition objects.
    stopConditions: typing.Dict[str, StopCondition] = {}

    #: A String specifying the optimization task repository key.
    name: str

    #: A Boolean specifying whether to use Abaqus to compute the design responses and their
    #: sensitivities. The default value is False.
    #:
    #: .. versionadded:: 2019
    #:     The `abaqusSensitivities` attribute was added.
    abaqusSensitivities: Boolean = False

    #: A SymbolicConstant specifying whether to control the permitted absolute step size by the
    #: average optimization displacement or minimum optimization displacement. Possible values
    #: are MINIMUM and AVERAGE. The default value is MINIMUM.
    absoluteStepSizeControl: SymbolicConstant = MINIMUM

    #: A boolean specifying whether or not the durability approach of optimization is turned
    #: on. The default value is ON.
    activateDurability: Boolean = ON

    #: A String specifying the path of additional files pertaining to durability optimization.
    #: Only valid if the **activateDurability** argument is ON.
    additionalDurabilityFiles: str = ""

    #: A SymbolicConstant specifying the constrained Laplacian convergence level. Possible
    #: values are NORMAL, CONSERVATIVE, and AGGRESSIVE. The default value is NORMAL.
    constrainedLaplacianConvergenceLevel: SymbolicConstant = NORMAL

    #: A Float specifying the edge length for the movement vector. The default value is 5.0.
    curvatureSmoothingEdgeLength: float = 5

    #: A string specifying the path of the input file. Only valid if the **activateDurability**
    #: argument is ON and is a required argument in that case.
    durabilityInputfile: str = ""

    #: A String specifying the type of solver for durability optimization. Possible values are:
    #: FE_SAFE, FEMFAT, FALANCS, MSC_FATIGUE, FE_FATIGUE, DESIGN_LIFE, CUSTOM, FEMSITE. The
    #: default value is FE_SAFE. Only valid if the **activateDurability** argument is ON.
    durabilitySolver: str = FE_SAFE

    #: A Float specifying the equality constraint tolerance. The default value is 10-3.
    equalityConstraintTolerance: float = None

    #: A Float specifying the mesh smoothing feature recognition angle for edges and corners.
    #: The default value is 30.0.
    featureRecognitionAngle: float = 30

    #: A Float specifying the weight depending on the radius, used when **filterMaxRadius** is
    #: specified. The default value is 1.0.
    filterExponent: float = 1

    #: None or a Float specifying the maximum influence radius for equivalent stress. The
    #: default value is None.
    filterMaxRadius: float = None

    #: None or a Float specifying the reduction of the radius depending on surface bending,
    #: used when **filterMaxRadius** is specified. The default value is None.
    filterRadiusReduction: float = None

    #: A SymbolicConstant specifying the method of specifying volume that can be removed
    #: immediately in the first design cycle. Possible values are OFF, PERCENTAGE, and
    #: ABSOLUTE. The default value is OFF.
    firstCycleDeletedVolumeTechnique: SymbolicConstant = OFF

    #: A Boolean specifying whether to exclude nodes with boundary conditions from the
    #: optimization. The default value is OFF.
    freezeBoundaryConditionRegions: Boolean = OFF

    #: The SymbolicConstant MODEL or a Region object specifying the region in which to freeze
    #: boundary condition regions, or the SymbolicConstant MODEL, used with
    #: **freezeBoundaryConditionRegions**. The default value is MODEL.
    frozenBoundaryConditionRegion: SymbolicConstant = MODEL

    #: A SymbolicConstant specifying the frequency of evaluating geometric restrictions during
    #: mesh smoothing. Possible values are LOW, MEDIUM, and HIGH. The default value is LOW.
    geometricRestrictionEvaluationFrequency: SymbolicConstant = LOW

    #: A Float specifying the scale factor to apply to optimization displacements for nodes
    #: with growth. The default value is 1.0.
    growthScaleFactor: float = 1

    #: A Boolean specifying whether to halt the optimization if quality criteria are not
    #: satisified. The default value is OFF.
    haltUponViolation: Boolean = OFF

    #: None or a Region object specifying the region specifying the first node layer for mesh
    #: smoothing, used when **meshSmoothingRegionMethod** is TASK_REGION_LAYERS. The default
    #: value is None.
    layerReferenceRegion: str = None

    #: A SymbolicConstant specifying the method used to determine the mesh smoothing region.
    #: The REGION value uses the **smoothingRegion**. The NUMBER_OF_LAYERS value uses the
    #: **layerReferenceRegion**. The TASK_REGION_LAYERS value will smooth six layers using the
    #: task region. Possible values are TASK_REGION_LAYERS, REGION, and NUMBER_OF_LAYERS. The
    #: default value is TASK_REGION_LAYERS.
    meshSmoothingRegionMethod: SymbolicConstant = TASK_REGION_LAYERS

    #: A SymbolicConstant specifying the method smoothing strategy. Possible values are
    #: CONSTRAINED_LAPLACIAN and LOCAL_GRADIENT. The default value is CONSTRAINED_LAPLACIAN.
    meshSmoothingStrategy: SymbolicConstant = CONSTRAINED_LAPLACIAN

    #: A SymbolicConstant specifying the approach used when treating midside node positions
    #: during optimization. POSITIONS indicates midside node positions are interpolated
    #: linearly by position. OPTIMIZATION_DISPLACEMENT indicates they are interpolated by
    #: optimization displacement of corner nodes. Possible values are POSITIONS and
    #: OPTIMIZATION_DISPLACEMENT. The default value is POSITIONS.
    midsideInterpolation: SymbolicConstant = POSITIONS

    #: The SymbolicConstant FIX_NONE or an Int specifying the number of node layers adjoining
    #: the task region to remain free during mesh smoothing. A value of 0 indicates that no
    #: layers are free and all layers are fixed. The default value is 0.
    numFreeNodeLayers: SymbolicConstant = 0

    #: None or an Int specifying the number of layers for mesh smoothing when
    #: **meshSmoothingRegionMethod** is NUMBER_OF_LAYERS. The default value is None.
    numSmoothedElementLayers: int = None

    #: A Boolean specifying whether to ignore automatically frozen boundary condition regions
    #: in the first design cycle. This is used with **freezeBoundaryConditionRegions**. The
    #: default value is ON.
    presumeFeasibleBCRegionAtStart: Boolean = ON

    #: A Float specifying the maximum angle for quad elements during mesh smoothing. The
    #: default value is 160.0.
    quadMaxAngle: float = 160

    #: A Float specifying the minimum angle for quad elements during mesh smoothing. The
    #: default value is 20.0.
    quadMinAngle: float = 20

    #: A Float specifying the skew angle for quad elements during mesh smoothing, used with
    #: **reportQualityViolation**. The default value is 30.0.
    quadSkew: float = 30

    #: A Float specifying the taper for quad elements during mesh smoothing, used with
    #: **reportQualityViolation**. The default value is 0.5.
    quadTaper: float = 0

    #: The SymbolicConstant MODEL or a Region object specifying the region to which the
    #: optimization task is applied. The default value is MODEL.
    region: SymbolicConstant = MODEL

    #: A Boolean specifying whether to report poor quality elements during mesh smoothing. The
    #: default value is OFF.
    reportPoorQualityElements: Boolean = OFF

    #: A Boolean specifying whether to report a quality criteria violation during mesh
    #: smoothing. The default value is OFF.
    reportQualityViolation: Boolean = OFF

    #: A Float specifying the scale factor to apply to optimization displacements for nodes
    #: with shrinkage. The default value is 1.0.
    shrinkScaleFactor: float = 1

    #: None or a Region object specifying the mesh smoothing region, used when
    #: **meshSmoothingRegionMethod** is REGION. The default value is None.
    smoothingRegion: str = None

    #: A SymbolicConstant specifying the target mesh quality for mesh smoothing. Possible
    #: values are NONE, LOW, MEDIUM, and HIGH. The default value is LOW.
    targetMeshQuality: SymbolicConstant = LOW

    #: A Float specifying the tet element aspect ratio during mesh smoothing. The default value
    #: is 100.0.
    tetAspectRatio: float = 100

    #: A Float specifying the maximum tet element aspect ratio during mesh smoothing. The
    #: default value is 8.0.
    tetMaxAspect: float = 8

    #: A Float specifying the minimum tet element aspect ratio during mesh smoothing. The
    #: default value is 0.222.
    tetMinAspect: float = 0

    #: A Float specifying the tet element skew value during mesh smoothing. The default value
    #: is 100.0.
    tetSkew: float = 100

    #: A Float specifying the tri element maximum angle during mesh smoothing. The default
    #: value is 140.0.
    triMaxAngle: float = 140

    #: A Float specifying the tri element maximum angle during mesh smoothing. The default
    #: value is 20.0.
    triMinAngle: float = 20

    #: A SymbolicConstant specifying whether to update shape basis vectors in the first design
    #: cycle or every design cycle. Possible values are EVERY_CYCLE and FIRST_CYCLE. The
    #: default value is EVERY_CYCLE.
    updateShapeBasisVectors: SymbolicConstant = EVERY_CYCLE

    @abaqus_method_doc
    def __init__(
        self,
        name: str,
        abaqusSensitivities: Boolean = True,
        absoluteStepSizeControl: SymbolicConstant = MINIMUM,
        activateDurability: Boolean = ON,
        additionalDurabilityFiles: str = "",
        constrainedLaplacianConvergenceLevel: SymbolicConstant = NORMAL,
        curvatureSmoothingEdgeLength: float = 5,
        durabilityInputfile: str = "",
        durabilitySolver: str = FE_SAFE,
        equalityConstraintTolerance: float = None,
        featureRecognitionAngle: float = 30,
        filterExponent: float = 1,
        filterMaxRadius: float = None,
        filterRadiusReduction: float = None,
        firstCycleDeletedVolumeTechnique: SymbolicConstant = OFF,
        freezeBoundaryConditionRegions: Boolean = OFF,
        frozenBoundaryConditionRegion: SymbolicConstant = MODEL,
        geometricRestrictionEvaluationFrequency: SymbolicConstant = LOW,
        growthScaleFactor: float = 1,
        haltUponViolation: Boolean = OFF,
        layerReferenceRegion: str = None,
        meshSmoothingRegionMethod: SymbolicConstant = TASK_REGION_LAYERS,
        meshSmoothingStrategy: SymbolicConstant = CONSTRAINED_LAPLACIAN,
        midsideInterpolation: SymbolicConstant = POSITIONS,
        numFreeNodeLayers: SymbolicConstant = 0,
        numSmoothedElementLayers: int = None,
        presumeFeasibleBCRegionAtStart: Boolean = ON,
        quadMaxAngle: float = 160,
        quadMinAngle: float = 20,
        quadSkew: float = 30,
        quadTaper: float = 0,
        region: SymbolicConstant = MODEL,
        reportPoorQualityElements: Boolean = OFF,
        reportQualityViolation: Boolean = OFF,
        shrinkScaleFactor: float = 1,
        smoothingRegion: str = None,
        targetMeshQuality: SymbolicConstant = LOW,
        tetAspectRatio: float = 100,
        tetMaxAspect: float = 8,
        tetMinAspect: float = 0,
        tetSkew: float = 100,
        triMaxAngle: float = 140,
        triMinAngle: float = 20,
        updateShapeBasisVectors: SymbolicConstant = EVERY_CYCLE,
    ):
        """This method creates a ShapeTask object.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].ShapeTask

        Parameters
        ----------
        name
            A String specifying the optimization task repository key.
        abaqusSensitivities
            A Boolean specifying whether to use Abaqus to compute the design responses and their
            sensitivities. The default value is True.

            .. versionadded:: 2019
                The `abaqusSensitivities` argument was added.
        absoluteStepSizeControl
            A SymbolicConstant specifying whether to control the permitted absolute step size by the
            average optimization displacement or minimum optimization displacement. Possible values
            are MINIMUM and AVERAGE. The default value is MINIMUM.
        activateDurability
            A boolean specifying whether or not the durability approach of optimization is turned
            on. The default value is ON.
        additionalDurabilityFiles
            A String specifying the path of additional files pertaining to durability optimization.
            Only valid if the **activateDurability** argument is ON.
        constrainedLaplacianConvergenceLevel
            A SymbolicConstant specifying the constrained Laplacian convergence level. Possible
            values are NORMAL, CONSERVATIVE, and AGGRESSIVE. The default value is NORMAL.
        curvatureSmoothingEdgeLength
            A Float specifying the edge length for the movement vector. The default value is 5.0.
        durabilityInputfile
            A string specifying the path of the input file. Only valid if the **activateDurability**
            argument is ON and is a required argument in that case.
        durabilitySolver
            A String specifying the type of solver for durability optimization. Possible values are:
            FE_SAFE, FEMFAT, FALANCS, MSC_FATIGUE, FE_FATIGUE, DESIGN_LIFE, CUSTOM, FEMSITE. The
            default value is FE_SAFE. Only valid if the **activateDurability** argument is ON.
        equalityConstraintTolerance
            A Float specifying the equality constraint tolerance. The default value is 10-3.
        featureRecognitionAngle
            A Float specifying the mesh smoothing feature recognition angle for edges and corners.
            The default value is 30.0.
        filterExponent
            A Float specifying the weight depending on the radius, used when **filterMaxRadius** is
            specified. The default value is 1.0.
        filterMaxRadius
            None or a Float specifying the maximum influence radius for equivalent stress. The
            default value is None.
        filterRadiusReduction
            None or a Float specifying the reduction of the radius depending on surface bending,
            used when **filterMaxRadius** is specified. The default value is None.
        firstCycleDeletedVolumeTechnique
            A SymbolicConstant specifying the method of specifying volume that can be removed
            immediately in the first design cycle. Possible values are OFF, PERCENTAGE, and
            ABSOLUTE. The default value is OFF.
        freezeBoundaryConditionRegions
            A Boolean specifying whether to exclude nodes with boundary conditions from the
            optimization. The default value is OFF.
        frozenBoundaryConditionRegion
            The SymbolicConstant MODEL or a Region object specifying the region in which to freeze
            boundary condition regions, or the SymbolicConstant MODEL, used with
            **freezeBoundaryConditionRegions**. The default value is MODEL.
        geometricRestrictionEvaluationFrequency
            A SymbolicConstant specifying the frequency of evaluating geometric restrictions during
            mesh smoothing. Possible values are LOW, MEDIUM, and HIGH. The default value is LOW.
        growthScaleFactor
            A Float specifying the scale factor to apply to optimization displacements for nodes
            with growth. The default value is 1.0.
        haltUponViolation
            A Boolean specifying whether to halt the optimization if quality criteria are not
            satisified. The default value is OFF.
        layerReferenceRegion
            None or a Region object specifying the region specifying the first node layer for mesh
            smoothing, used when **meshSmoothingRegionMethod** is TASK_REGION_LAYERS. The default
            value is None.
        meshSmoothingRegionMethod
            A SymbolicConstant specifying the method used to determine the mesh smoothing region.
            The REGION value uses the **smoothingRegion**. The NUMBER_OF_LAYERS value uses the
            **layerReferenceRegion**. The TASK_REGION_LAYERS value will smooth six layers using the
            task region. Possible values are TASK_REGION_LAYERS, REGION, and NUMBER_OF_LAYERS. The
            default value is TASK_REGION_LAYERS.
        meshSmoothingStrategy
            A SymbolicConstant specifying the method smoothing strategy. Possible values are
            CONSTRAINED_LAPLACIAN and LOCAL_GRADIENT. The default value is CONSTRAINED_LAPLACIAN.
        midsideInterpolation
            A SymbolicConstant specifying the approach used when treating midside node positions
            during optimization. POSITIONS indicates midside node positions are interpolated
            linearly by position. OPTIMIZATION_DISPLACEMENT indicates they are interpolated by
            optimization displacement of corner nodes. Possible values are POSITIONS and
            OPTIMIZATION_DISPLACEMENT. The default value is POSITIONS.
        numFreeNodeLayers
            The SymbolicConstant FIX_NONE or an Int specifying the number of node layers adjoining
            the task region to remain free during mesh smoothing. A value of 0 indicates that no
            layers are free and all layers are fixed. The default value is 0.
        numSmoothedElementLayers
            None or an Int specifying the number of layers for mesh smoothing when
            **meshSmoothingRegionMethod** is NUMBER_OF_LAYERS. The default value is None.
        presumeFeasibleBCRegionAtStart
            A Boolean specifying whether to ignore automatically frozen boundary condition regions
            in the first design cycle. This is used with **freezeBoundaryConditionRegions**. The
            default value is ON.
        quadMaxAngle
            A Float specifying the maximum angle for quad elements during mesh smoothing. The
            default value is 160.0.
        quadMinAngle
            A Float specifying the minimum angle for quad elements during mesh smoothing. The
            default value is 20.0.
        quadSkew
            A Float specifying the skew angle for quad elements during mesh smoothing, used with
            **reportQualityViolation**. The default value is 30.0.
        quadTaper
            A Float specifying the taper for quad elements during mesh smoothing, used with
            **reportQualityViolation**. The default value is 0.5.
        region
            The SymbolicConstant MODEL or a Region object specifying the region to which the
            optimization task is applied. The default value is MODEL.
        reportPoorQualityElements
            A Boolean specifying whether to report poor quality elements during mesh smoothing. The
            default value is OFF.
        reportQualityViolation
            A Boolean specifying whether to report a quality criteria violation during mesh
            smoothing. The default value is OFF.
        shrinkScaleFactor
            A Float specifying the scale factor to apply to optimization displacements for nodes
            with shrinkage. The default value is 1.0.
        smoothingRegion
            None or a Region object specifying the mesh smoothing region, used when
            **meshSmoothingRegionMethod** is REGION. The default value is None.
        targetMeshQuality
            A SymbolicConstant specifying the target mesh quality for mesh smoothing. Possible
            values are NONE, LOW, MEDIUM, and HIGH. The default value is LOW.
        tetAspectRatio
            A Float specifying the tet element aspect ratio during mesh smoothing. The default value
            is 100.0.
        tetMaxAspect
            A Float specifying the maximum tet element aspect ratio during mesh smoothing. The
            default value is 8.0.
        tetMinAspect
            A Float specifying the minimum tet element aspect ratio during mesh smoothing. The
            default value is 0.222.
        tetSkew
            A Float specifying the tet element skew value during mesh smoothing. The default value
            is 100.0.
        triMaxAngle
            A Float specifying the tri element maximum angle during mesh smoothing. The default
            value is 140.0.
        triMinAngle
            A Float specifying the tri element maximum angle during mesh smoothing. The default
            value is 20.0.
        updateShapeBasisVectors
            A SymbolicConstant specifying whether to update shape basis vectors in the first design
            cycle or every design cycle. Possible values are EVERY_CYCLE and FIRST_CYCLE. The
            default value is EVERY_CYCLE.

        Returns
        -------
        ShapeTask
            A :py:class:`~abaqus.Optimization.ShapeTask.ShapeTask` object.
        """
        super().__init__()

    @abaqus_method_doc
    def setValues(
        self,
        abaqusSensitivities: Boolean = True,
        absoluteStepSizeControl: SymbolicConstant = MINIMUM,
        activateDurability: Boolean = ON,
        additionalDurabilityFiles: str = "",
        algorithm: SymbolicConstant = CONDITION_BASED_OPTIMIZATION,
        constrainedLaplacianConvergenceLevel: SymbolicConstant = NORMAL,
        curvatureSmoothingEdgeLength: float = 5,
        durabilityInputfile: str = "",
        durabilitySolver: str = FE_SAFE,
        equalityConstraintTolerance: float = None,
        featureRecognitionAngle: float = 30,
        filterExponent: float = 1,
        filterMaxRadius: float = None,
        filterRadiusReduction: float = None,
        firstCycleDeletedVolumeTechnique: SymbolicConstant = OFF,
        freezeBoundaryConditionRegions: Boolean = OFF,
        frozenBoundaryConditionRegion: SymbolicConstant = MODEL,
        geometricRestrictionEvaluationFrequency: SymbolicConstant = LOW,
        growthScaleFactor: float = 1,
        haltUponViolation: Boolean = OFF,
        layerReferenceRegion: str = None,
        meshSmoothingRegionMethod: SymbolicConstant = TASK_REGION_LAYERS,
        meshSmoothingStrategy: SymbolicConstant = CONSTRAINED_LAPLACIAN,
        midsideInterpolation: SymbolicConstant = POSITIONS,
        numFreeNodeLayers: SymbolicConstant = 0,
        numSmoothedElementLayers: int = None,
        presumeFeasibleBCRegionAtStart: Boolean = ON,
        quadMaxAngle: float = 160,
        quadMinAngle: float = 20,
        quadSkew: float = 30,
        quadTaper: float = 0,
        region: SymbolicConstant = MODEL,
        reportPoorQualityElements: Boolean = OFF,
        reportQualityViolation: Boolean = OFF,
        shrinkScaleFactor: float = 1,
        smoothingRegion: str = None,
        targetMeshQuality: SymbolicConstant = LOW,
        tetAspectRatio: float = 100,
        tetMaxAspect: float = 8,
        tetMinAspect: float = 0,
        tetSkew: float = 100,
        triMaxAngle: float = 140,
        triMinAngle: float = 20,
        updateShapeBasisVectors: SymbolicConstant = EVERY_CYCLE,
    ):
        """This method modifies the ShapeTask object.

        Parameters
        ----------
        abaqusSensitivities
            A Boolean specifying whether to use Abaqus to compute the design responses and their
            sensitivities. The default value is True.

            .. versionadded:: 2019
                The `abaqusSensitivities` argument was added.
        absoluteStepSizeControl
            A SymbolicConstant specifying whether to control the permitted absolute step size by the
            average optimization displacement or minimum optimization displacement. Possible values
            are MINIMUM and AVERAGE. The default value is MINIMUM.
        activateDurability
            A boolean specifying whether or not the durability approach of optimization is turned
            on. The default value is ON.
        additionalDurabilityFiles
            A String specifying the path of additional files pertaining to durability optimization.
            Only valid if the **activateDurability** argument is ON.
        algorithm
            A SymbolicConstant specifying the optimization task algorithm. Possible values are
            GENERAL_OPTIMIZATION and CONDITION_BASED_OPTIMIZATION. The default value is
            CONDITION_BASED_OPTIMIZATION.
        constrainedLaplacianConvergenceLevel
            A SymbolicConstant specifying the constrained Laplacian convergence level. Possible
            values are NORMAL, CONSERVATIVE, and AGGRESSIVE. The default value is NORMAL.
        curvatureSmoothingEdgeLength
            A Float specifying the edge length for the movement vector. The default value is 5.0.
        durabilityInputfile
            A string specifying the path of the input file. Only valid if the **activateDurability**
            argument is ON and is a required argument in that case.
        durabilitySolver
            A String specifying the type of solver for durability optimization. Possible values are:
            FE_SAFE, FEMFAT, FALANCS, MSC_FATIGUE, FE_FATIGUE, DESIGN_LIFE, CUSTOM, FEMSITE. The
            default value is FE_SAFE. Only valid if the **activateDurability** argument is ON.
        equalityConstraintTolerance
            A Float specifying the equality constraint tolerance. The default value is 10-3.
        featureRecognitionAngle
            A Float specifying the mesh smoothing feature recognition angle for edges and corners.
            The default value is 30.0.
        filterExponent
            A Float specifying the weight depending on the radius, used when **filterMaxRadius** is
            specified. The default value is 1.0.
        filterMaxRadius
            None or a Float specifying the maximum influence radius for equivalent stress. The
            default value is None.
        filterRadiusReduction
            None or a Float specifying the reduction of the radius depending on surface bending,
            used when **filterMaxRadius** is specified. The default value is None.
        firstCycleDeletedVolumeTechnique
            A SymbolicConstant specifying the method of specifying volume that can be removed
            immediately in the first design cycle. Possible values are OFF, PERCENTAGE, and
            ABSOLUTE. The default value is OFF.
        freezeBoundaryConditionRegions
            A Boolean specifying whether to exclude nodes with boundary conditions from the
            optimization. The default value is OFF.
        frozenBoundaryConditionRegion
            The SymbolicConstant MODEL or a Region object specifying the region in which to freeze
            boundary condition regions, or the SymbolicConstant MODEL, used with
            **freezeBoundaryConditionRegions**. The default value is MODEL.
        geometricRestrictionEvaluationFrequency
            A SymbolicConstant specifying the frequency of evaluating geometric restrictions during
            mesh smoothing. Possible values are LOW, MEDIUM, and HIGH. The default value is LOW.
        growthScaleFactor
            A Float specifying the scale factor to apply to optimization displacements for nodes
            with growth. The default value is 1.0.
        haltUponViolation
            A Boolean specifying whether to halt the optimization if quality criteria are not
            satisified. The default value is OFF.
        layerReferenceRegion
            None or a Region object specifying the region specifying the first node layer for mesh
            smoothing, used when **meshSmoothingRegionMethod** is TASK_REGION_LAYERS. The default
            value is None.
        meshSmoothingRegionMethod
            A SymbolicConstant specifying the method used to determine the mesh smoothing region.
            The REGION value uses the **smoothingRegion**. The NUMBER_OF_LAYERS value uses the
            **layerReferenceRegion**. The TASK_REGION_LAYERS value will smooth six layers using the
            task region. Possible values are TASK_REGION_LAYERS, REGION, and NUMBER_OF_LAYERS. The
            default value is TASK_REGION_LAYERS.
        meshSmoothingStrategy
            A SymbolicConstant specifying the method smoothing strategy. Possible values are
            CONSTRAINED_LAPLACIAN and LOCAL_GRADIENT. The default value is CONSTRAINED_LAPLACIAN.
        midsideInterpolation
            A SymbolicConstant specifying the approach used when treating midside node positions
            during optimization. POSITIONS indicates midside node positions are interpolated
            linearly by position. OPTIMIZATION_DISPLACEMENT indicates they are interpolated by
            optimization displacement of corner nodes. Possible values are POSITIONS and
            OPTIMIZATION_DISPLACEMENT. The default value is POSITIONS.
        numFreeNodeLayers
            The SymbolicConstant FIX_NONE or an Int specifying the number of node layers adjoining
            the task region to remain free during mesh smoothing. A value of 0 indicates that no
            layers are free and all layers are fixed. The default value is 0.
        numSmoothedElementLayers
            None or an Int specifying the number of layers for mesh smoothing when
            **meshSmoothingRegionMethod** is NUMBER_OF_LAYERS. The default value is None.
        presumeFeasibleBCRegionAtStart
            A Boolean specifying whether to ignore automatically frozen boundary condition regions
            in the first design cycle. This is used with **freezeBoundaryConditionRegions**. The
            default value is ON.
        quadMaxAngle
            A Float specifying the maximum angle for quad elements during mesh smoothing. The
            default value is 160.0.
        quadMinAngle
            A Float specifying the minimum angle for quad elements during mesh smoothing. The
            default value is 20.0.
        quadSkew
            A Float specifying the skew angle for quad elements during mesh smoothing, used with
            **reportQualityViolation**. The default value is 30.0.
        quadTaper
            A Float specifying the taper for quad elements during mesh smoothing, used with
            **reportQualityViolation**. The default value is 0.5.
        region
            The SymbolicConstant MODEL or a Region object specifying the region to which the
            optimization task is applied. The default value is MODEL.
        reportPoorQualityElements
            A Boolean specifying whether to report poor quality elements during mesh smoothing. The
            default value is OFF.
        reportQualityViolation
            A Boolean specifying whether to report a quality criteria violation during mesh
            smoothing. The default value is OFF.
        shrinkScaleFactor
            A Float specifying the scale factor to apply to optimization displacements for nodes
            with shrinkage. The default value is 1.0.
        smoothingRegion
            None or a Region object specifying the mesh smoothing region, used when
            **meshSmoothingRegionMethod** is REGION. The default value is None.
        targetMeshQuality
            A SymbolicConstant specifying the target mesh quality for mesh smoothing. Possible
            values are NONE, LOW, MEDIUM, and HIGH. The default value is LOW.
        tetAspectRatio
            A Float specifying the tet element aspect ratio during mesh smoothing. The default value
            is 100.0.
        tetMaxAspect
            A Float specifying the maximum tet element aspect ratio during mesh smoothing. The
            default value is 8.0.
        tetMinAspect
            A Float specifying the minimum tet element aspect ratio during mesh smoothing. The
            default value is 0.222.
        tetSkew
            A Float specifying the tet element skew value during mesh smoothing. The default value
            is 100.0.
        triMaxAngle
            A Float specifying the tri element maximum angle during mesh smoothing. The default
            value is 140.0.
        triMinAngle
            A Float specifying the tri element maximum angle during mesh smoothing. The default
            value is 20.0.
        updateShapeBasisVectors
            A SymbolicConstant specifying whether to update shape basis vectors in the first design
            cycle or every design cycle. Possible values are EVERY_CYCLE and FIRST_CYCLE. The
            default value is EVERY_CYCLE.
        """
        ...



import typing

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from .Cell import Cell
from ..UtilityAndView.abaqusConstants import *


@abaqus_class_doc
class CellArray(typing.List[Cell]):
    """The CellArray is a sequence of Cell objects.

    .. note:: 
        This object can be accessed by::

            import part
            mdb.models[name].parts[name].allInternalSets[name].cells
            mdb.models[name].parts[name].allSets[name].cells
            mdb.models[name].parts[name].cells
            mdb.models[name].parts[name].sets[name].cells
            import assembly
            mdb.models[name].rootAssembly.allInstances[name].cells
            mdb.models[name].rootAssembly.allInstances[name].sets[name].cells
            mdb.models[name].rootAssembly.allInternalSets[name].cells
            mdb.models[name].rootAssembly.allSets[name].cells
            mdb.models[name].rootAssembly.instances[name].cells
            mdb.models[name].rootAssembly.instances[name].sets[name].cells
            mdb.models[name].rootAssembly.modelInstances[i].sets[name].cells
            mdb.models[name].rootAssembly.sets[name].cells
    """

    @abaqus_method_doc
    def __init__(self, cells: typing.List[Cell]):
        """This method creates a CellArray object.

        .. note:: 
            This function can be accessed by::

                part.CellArray

        Parameters
        ----------
        cells
            A list of Cell objects.

        Returns
        -------
        CellArray
            A :py:class:`~abaqus.BasicGeometry.CellArray.CellArray` object.

        """
        ...

    @abaqus_method_doc
    def findAt(self, coordinates: tuple, printWarning: Boolean = True) -> typing.Union[Cell, typing.List[Cell]]:
        """This method returns the object or objects in the CellArray located at the given
        coordinates. findAt initially uses the ACIS tolerance of 1E-6. As a result, findAt
        returns any entity that is at the arbitrary point specified or at a distance of less
        than 1E-6 from the arbitrary point. If nothing is found, findAt uses the tolerance for
        imprecise geometry (applicable only for imprecise geometric entities). The arbitrary
        point must not be shared by a second cell. If two cells intersect or coincide at the
        arbitrary point, findAt chooses the first cell that it encounters, and you should not
        rely on the return value being consistent.
        findAt will always try to find objects among all the cells in the part or assembly
        instance and will not restrict itself to a subset even if the CellArray represents such
        subset.

        Parameters
        ----------
        coordinates
            A sequence of Floats specifying the **X**-, **Y**-, and **Z**-coordinates of the object to
            find.findAt returns either a Cell object or a sequence of Cell objects based on the type
            of input. If **coordinates** is a sequence of Floats, findAt returns the Cell object at
            that point. If **coordinates** is a sequence of sequence of Floats, findAt returns a
            sequence of Cell objects at the given locations. The sequence of sequence of Floats must
            be a sequence of sequence of point and normal data, not a sequence of point data. For
            example::
            
                cells1 = myCrackedBlockInstance.cells.findAt(((5.5, -8.3, 294.2),), 
                                                             ((12.1, -8.3, 287.8),), 
                                                             ((14.4, -10.4, 285.5),),)
        printWarning
            A Boolean specifying whether a message is to be printed to the CLI if no entity is found
            at the specified location. The default value is True.

        Returns
        -------
        Cell
            A :py:class:`~abaqus.BasicGeometry.Cell.Cell` object.

        """
        return Cell()

    @abaqus_method_doc
    def getExteriorFaces(self):
        """This method returns the cell faces on the exterior of the CellArray. That is, it returns
        the faces that are referenced by exactly one of the cells in the sequence.

        Returns
        -------
        FaceArray
            A :py:class:`~abaqus.BasicGeometry.FaceArray.FaceArray` object representing the faces on the exterior of the cells.

        """
        ...

    @abaqus_method_doc
    def getSequenceFromMask(self, mask: str):
        """This method returns the object or objects in the CellArray identified using the
        specified **mask**. This command is generated when the JournalOptions are set to
        COMPRESSEDINDEX. When large number of objects are involved, this method is highly
        efficient.

        Parameters
        ----------
        mask
            A String specifying the object or objects.

        Returns
        -------
        Cell
            A :py:class:`~abaqus.BasicGeometry.Cell.Cell` object or a sequence of Cell objects.

        Raises
        ------
        Error: The mask results in an empty sequence
            An exception occurs if the resulting sequence is empty.
        """
        ...

    @abaqus_method_doc
    def getMask(self):
        """This method returns a string specifying the object or objects.

        Returns
        -------
        str
            A String specifying the object or objects.

        """
        ...

    @abaqus_method_doc
    def getByBoundingBox(
        self,
        xMin: str = "",
        yMin: str = "",
        zMin: str = "",
        xMax: str = "",
        yMax: str = "",
        zMax: str = "",
    ):
        """This method returns an array of cell objects that lie within the specified bounding box.

        Parameters
        ----------
        xMin
            A float specifying the minimum **X**-boundary of the bounding box.
        yMin
            A float specifying the minimum **Y**-boundary of the bounding box.
        zMin
            A float specifying the minimum **Z**-boundary of the bounding box.
        xMax
            A float specifying the maximum **X**-boundary of the bounding box.
        yMax
            A float specifying the maximum **Y**-boundary of the bounding box.
        zMax
            A float specifying the maximum **Z**-boundary of the bounding box.

        Returns
        -------
        CellArray
            A :py:class:`~abaqus.BasicGeometry.CellArray.CellArray` object, which is a sequence of Cell objects.

        """
        ...

    @abaqus_method_doc
    def getByBoundingCylinder(self, center1: tuple, center2: tuple, radius: str):
        """This method returns an array of cell objects that lie within the specified bounding
        cylinder.

        Parameters
        ----------
        center1
            A tuple of the **X**-, **Y**-, and **Z**-coordinates of the center of the first end of the
            cylinder.
        center2
            A tuple of the **X**-, **Y**-, and **Z**-coordinates of the center of the second end of the
            cylinder.
        radius
            A float specifying the radius of the cylinder.

        Returns
        -------
        CellArray
            A :py:class:`~abaqus.BasicGeometry.CellArray.CellArray` object, which is a sequence of Cell objects.

        """
        ...

    @abaqus_method_doc
    def getByBoundingSphere(self, center: tuple, radius: str):
        """This method returns an array of cell objects that lie within the specified bounding
        sphere.

        Parameters
        ----------
        center
            A tuple of the **X**-, **Y**-, and **Z**-coordinates of the center of the sphere.
        radius
            A float specifying the radius of the sphere.

        Returns
        -------
        CellArray
            A :py:class:`~abaqus.BasicGeometry.CellArray.CellArray` object, which is a sequence of Cell objects.

        """
        ...

    @abaqus_method_doc
    def getBoundingBox(self):
        """This method returns a dictionary of two tuples representing minimum and maximum boundary
        values of the bounding box of the minimum size containing the cell sequence.

        Returns
        -------
        typing.Dict[str, typing.Tuple[float, float, float]]
            A Dictionary object with the following items:
            
            - **low**: a tuple of three floats representing the minimum **X** -, **Y** -, and **Z** -boundary
              values of the bounding box.
            - **high**: a tuple of three floats representing the maximum **X** -, **Y** -, and **Z** -boundary
              values of the bounding box.
        """
        ...



import typing

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from .AnalysisStep import AnalysisStep
from ..Adaptivity.AdaptiveMeshConstraintState import AdaptiveMeshConstraintState
from ..Adaptivity.AdaptiveMeshDomain import AdaptiveMeshDomain
from ..BoundaryCondition.BoundaryConditionState import BoundaryConditionState
from ..Load.LoadCase import LoadCase
from ..Load.LoadState import LoadState
from ..PredefinedField.PredefinedFieldState import PredefinedFieldState
from ..StepMiscellaneous.Control import Control
from ..StepMiscellaneous.SolverControl import SolverControl
from ..StepOutput.DiagnosticPrint import DiagnosticPrint
from ..StepOutput.FieldOutputRequestState import FieldOutputRequestState
from ..StepOutput.HistoryOutputRequestState import HistoryOutputRequestState
from ..StepOutput.Monitor import Monitor
from ..StepOutput.Restart import Restart
from ..UtilityAndView.abaqusConstants import *


@abaqus_class_doc
class AnnealStep(AnalysisStep):
    """The AnnealStep object anneals a structure by setting the velocities and all appropriate
    state variables to zero.
    The AnnealStep object is derived from the AnalysisStep object.

    .. note:: 
        This object can be accessed by::

            import step
            mdb.models[name].steps[name]

        The corresponding analysis keywords are:

        - ANNEAL
        - STEP
    """

    #: A String specifying the repository key.
    name: str = ""

    #: A Float specifying the post-anneal reference temperature. The default value is the
    #: current temperature at all nodes in the model after the annealing has completed.
    refTemp: float = None

    #: A String specifying the name of the previous step. The new step appears after this step
    #: in the list of analysis steps.
    previous: str = ""

    #: A String specifying a description of the new step. The default value is an empty string.
    description: str = ""

    #: A SymbolicConstant specifying whether the step has an explicit procedure type
    #: (*procedureType* = ANNEAL, DYNAMIC_EXPLICIT, or DYNAMIC_TEMP_DISPLACEMENT).
    explicit: SymbolicConstant = None

    #: A Boolean specifying whether the step has a perturbation procedure type.
    perturbation: Boolean = OFF

    #: A Boolean specifying whether the step has a mechanical procedure type.
    nonmechanical: Boolean = OFF

    #: A SymbolicConstant specifying the Abaqus procedure. Possible values are:
    #: 
    #: - ANNEAL
    #: - BUCKLE
    #: - COMPLEX_FREQUENCY
    #: - COUPLED_TEMP_DISPLACEMENT
    #: - COUPLED_THERMAL_ELECTRIC
    #: - DIRECT_CYCLIC
    #: - DYNAMIC_IMPLICIT
    #: - DYNAMIC_EXPLICIT
    #: - DYNAMIC_SUBSPACE
    #: - DYNAMIC_TEMP_DISPLACEMENT
    #: - COUPLED_THERMAL_ELECTRICAL_STRUCTURAL
    #: - FREQUENCY
    #: - GEOSTATIC
    #: - HEAT_TRANSFER
    #: - MASS_DIFFUSION
    #: - MODAL_DYNAMICS
    #: - RANDOM_RESPONSE
    #: - RESPONSE_SPECTRUM
    #: - SOILS
    #: - STATIC_GENERAL
    #: - STATIC_LINEAR_PERTURBATION
    #: - STATIC_RIKS
    #: - STEADY_STATE_DIRECT
    #: - STEADY_STATE_MODAL
    #: - STEADY_STATE_SUBSPACE
    #: - VISCO
    procedureType: SymbolicConstant = None

    #: A Boolean specifying whether the step is suppressed or not. The default value is OFF.
    suppressed: Boolean = OFF

    #: A repository of FieldOutputRequestState objects.
    fieldOutputRequestState: typing.Dict[str, FieldOutputRequestState] = {}

    #: A repository of HistoryOutputRequestState objects.
    historyOutputRequestState: typing.Dict[str, HistoryOutputRequestState] = {}

    #: A :py:class:`~abaqus.StepOutput.DiagnosticPrint.DiagnosticPrint` object.
    diagnosticPrint: DiagnosticPrint = DiagnosticPrint()

    #: A :py:class:`~abaqus.StepOutput.Monitor.Monitor` object.
    monitor: Monitor = None

    #: A :py:class:`~abaqus.StepOutput.Restart.Restart` object.
    restart: Restart = Restart()

    #: A repository of AdaptiveMeshConstraintState objects.
    adaptiveMeshConstraintStates: typing.Dict[str, AdaptiveMeshConstraintState] = {}

    #: A repository of AdaptiveMeshDomain objects.
    adaptiveMeshDomains: typing.Dict[str, AdaptiveMeshDomain] = {}

    #: A :py:class:`~abaqus.StepMiscellaneous.Control.Control` object.
    control: Control = Control()

    #: A :py:class:`~abaqus.StepMiscellaneous.SolverControl.SolverControl` object.
    solverControl: SolverControl = SolverControl()

    #: A repository of BoundaryConditionState objects.
    boundaryConditionStates: typing.Dict[str, BoundaryConditionState] = {}

    #: A repository of InteractionState objects.
    interactionStates: int = None

    #: A repository of LoadState objects.
    loadStates: typing.Dict[str, LoadState] = {}

    #: A repository of LoadCase objects.
    loadCases: typing.Dict[str, LoadCase] = {}

    #: A repository of PredefinedFieldState objects.
    predefinedFieldStates: typing.Dict[str, PredefinedFieldState] = {}

    @abaqus_method_doc
    def __init__(
        self,
        name: str,
        previous: str,
        description: str = "",
        refTemp: float = None,
        maintainAttributes: Boolean = False,
    ):
        """This method creates an AnnealStep object.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].AnnealStep

        Parameters
        ----------
        name
            A String specifying the repository key.
        previous
            A String specifying the name of the previous step. The new step appears after this step
            in the list of analysis steps.
        description
            A String specifying a description of the new step. The default value is an empty string.
        refTemp
            A Float specifying the post-anneal reference temperature. The default value is the
            current temperature at all nodes in the model after the annealing has completed.
        maintainAttributes
            A Boolean specifying whether to retain attributes from an existing step with the same
            name. The default value is False.

        Returns
        -------
        AnnealStep
            An :py:class:`~abaqus.Step.AnnealStep.AnnealStep` object.

        Raises
        ------
        RangeError
        """
        super().__init__()

    @abaqus_method_doc
    def setValues(self, description: str = "", refTemp: float = None):
        """This method modifies the AnnealStep object.

        Parameters
        ----------
        description
            A String specifying a description of the new step. The default value is an empty string.
        refTemp
            A Float specifying the post-anneal reference temperature. The default value is the
            current temperature at all nodes in the model after the annealing has completed.

        Raises
        ------
        RangeError
        """
        ...



from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from ...UtilityAndView.abaqusConstants import *


@abaqus_class_doc
class GapConvection:
    """The GapConvection object specifies the Nusselt number (Nu) to calculate the convective
    coefficient for heat transfer between the gap flow and both the top and bottom surfaces
    of a coupled temperature-pore pressure cohesive element.

    .. note::
        This object can be accessed by::

            import material
            mdb.models[name].materials[name].gapConvection
            import odbMaterial
            session.odbs[name].materials[name].gapConvection

        The table data for this object are:
        For **type** = TABULAR the table data specify the following:

        - Nusselt number (Nu)
        - Temperature, if the data depend on temperature.
        - Value of the first field variable, if the data depend on field variables.
        - Value of the second field variable.
        - Etc.

        The corresponding analysis keywords are:

        - GAP CONVECTION

    .. versionadded:: 2021
        The `GapConvection` class was added.
    """

    @abaqus_method_doc
    def __init__(
        self,
        type: str,
        table: tuple = (),
        temperatureDependency: Boolean = OFF,
        dependencies: int = 0,
    ):
        """This method creates a GapConvection object.

        .. note::
            This function can be accessed by::

                mdb.models[name].materials[name].GapConvection
                session.odbs[name].materials[name].GapConvection

        Parameters
        ----------
        type
            An odb_String specifying the type of gap convection. Possible values are FLUX,
            TEMPERATURE, and TABULAR. The default value is FLUX.
        table
            A sequence of sequences of Floats specifying the items described below.
        temperatureDependency
            A Boolean specifying whether the data depend on temperature. The default value is OFF.
        dependencies
            An Int specifying the number of field variable dependencies. The default value is 0.

        Returns
        -------
            A GapConvection object.
        """
        ...

    @abaqus_method_doc
    def setValues(self):
        """This method modifies the GapConvection object.
        """
        ...



from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from .Area import Area
from .AreaStyle import AreaStyle
from .Axis import Axis
from .Legend import Legend
from .LineStyle import LineStyle
from .TextStyle import TextStyle
from ..UtilityAndView.abaqusConstants import *


@abaqus_class_doc
class DefaultChartOptions:
    """The DefaultChartOptions object is used to hold on default chart and axis attributes. The
    DefaultChartOptions object attributes are used whenever Chart or Axis are created. A
    DefaultChartOptions object is automatically created when opening a session.

    .. note:: 
        This object can be accessed by::

            import visualization
            session.defaultChartOptions
    """

    areaStyle: AreaStyle = AreaStyle()

    @abaqus_method_doc
    def setValues(
        self,
        areaStyle: AreaStyle,
        aspectRatio: float = None,
        defaultAxis1Options: Axis = None, 
        defaultAxis2Options: Axis = None, 
        gridArea: Area = None, 
        legend: Legend = None, 
        majorAxis1GridStyle: LineStyle = None, 
        majorAxis2GridStyle: LineStyle = None, 
        minorAxis1GridStyle: LineStyle = None, 
        minorAxis2GridStyle: LineStyle = None, 
        tagAreaStyle: AreaStyle = None, 
        tagBorder: LineStyle = None, 
        tagTextStyle: TextStyle = None, 
        useQuantityType: Boolean = ON,
    ):
        """This method modifies the DefaultChartOptions object.

        Parameters
        ----------
        areaStyle
            An :py:class:`~abaqus.XY.AreaStyle.AreaStyle` object specifying an AreaStyle used to hold on to the default display
            properties for the chart area.
        aspectRatio
            A Float specifying the default aspect ratio of the grid area. A value of -1 specifies
            that the gridArea will take up all available space. The default value is −1.
        defaultAxis1Options
            An :py:class:`~abaqus.XY.Axis.Axis` object specifying an Axis object used to hold on to the default properties for
            direction 1 axes—the abscissa for a Cartesian chart.
        defaultAxis2Options
            An :py:class:`~abaqus.XY.Axis.Axis` object specifying an Axis object used to hold on to the default properties for
            direction 2 axes—the ordinate for a Cartesian chart.
        gridArea
            An :py:class:`~abaqus.XY.Area.Area` object specifying how to display the grid area by default.
        legend
            A :py:class:`~abaqus.XY.Legend.Legend` object specifying the default attributes for the legend of the chart.
        majorAxis1GridStyle
            A :py:class:`~abaqus.XY.LineStyle.LineStyle` object specifying the default line properties to be used when drawing major
            gridlines along axis 1.
        majorAxis2GridStyle
            A :py:class:`~abaqus.XY.LineStyle.LineStyle` object specifying the default line properties to be used when drawing major
            gridlines along axis 2.
        minorAxis1GridStyle
            A :py:class:`~abaqus.XY.LineStyle.LineStyle` object specifying the default line properties to be used when drawing minor
            gridlines along axis 1.
        minorAxis2GridStyle
            A :py:class:`~abaqus.XY.LineStyle.LineStyle` object specifying the default line properties to be used when drawing minor
            gridlines along axis 2.
        tagAreaStyle
            An :py:class:`~abaqus.XY.AreaStyle.AreaStyle` object specifying the default area properties to be used when creating
            tags.
        tagBorder
            A :py:class:`~abaqus.XY.LineStyle.LineStyle` object specifying the default tag area border properties to be used when
            creating tags.
        tagTextStyle
            A :py:class:`~abaqus.XY.TextStyle.TextStyle` object specifying the default text properties to be used when creating tags.
        useQuantityType
            A Boolean specifying whether to use the QuantityType to associate curves with axes. The
            default value is ON.
        """
        ...



from abqpy.decorators import abaqus_class_doc
from ..UtilityAndView.abaqusConstants import *


@abaqus_class_doc
class OdbAuxiliaryData:
    """The OdbAuxiliaryData object stores auxiliary data related to the steps in the analysis.

    .. note:: 
        This object can be accessed by::

            import visualization
            session.odbData[name].diagnosticData.analysisErrors[i].data[i]
            session.odbData[name].diagnosticData.analysisWarnings[i].data[i]
            session.odbData[name].diagnosticData.steps[i].contactDiagnostics[i].data[i]
    """

    #: A tuple consisting of element instance names. This attribute is read-only.
    elementInstanceNames: tuple = ()

    #: A tuple consisting of element label names. This attribute is read-only.
    elementLabels: tuple = ()

    #: A boolean specifying whether the OdbAuxiliaryData object has values in it. This
    #: attribute is read-only.
    hasValues: Boolean = OFF

    #: A tuple consisting of node instance names. This attribute is read-only.
    nodeInstanceNames: tuple = ()

    #: A tuple consisting of node label names. This attribute is read-only.
    nodeLables: tuple = ()



import typing

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from ..UtilityAndView.abaqusConstants import *


@abaqus_class_doc
class SurfaceVertexCriteriaAssignment:
    """The SurfaceVertexCriteriaAssignment object stores the surface vertex criteria assignment
    definition for surfaces in ContactStd objects. The SurfaceVertexCriteriaAssignment
    object has no constructor or members.

    .. note::
        This object can be accessed by::

            import interaction
            mdb.models[name].interactions[name].surfacevertexCriteriaAssignments

        The corresponding analysis keywords are:

        - SURFACE PROPERTY ASSIGNMENT

    .. versionadded:: 2021
        The `SurfaceVertexCriteriaAssignment` class was added.
    """

    @abaqus_method_doc
    def changeValuesInStep(
        self, stepName: str, index: int, value: typing.Union[SymbolicConstant, float]
    ):
        """This method allows modification of surface vertex criteria assignments already defined
        on surfaces in a given step.

        Parameters
        ----------
        stepName
            A String specifying the name of the step in which the surface vertex criteria
            assignments are to be modified.
        index
            An Int specifying the position of the surface vertex criteria assignment whose value is
            to be modified.
        value
            A tuple specifying the value of the surface vertex criteria assignments for the surface
            whose index is referenced. Each tuple contains:
            - A Float or a SymbolicConstant specifying the vertex criteria value to be used for the
            surface. Possible values of the SymbolicConstant are ALL_VERTICES or NO_VERTICES.
        """
        ...

    @abaqus_method_doc
    def appendInStep(
        self, stepName: str, assignments: typing.Union[SymbolicConstant, float]
    ):
        """This method allows addition of surface vertex criteria assignments to new surfaces in a
        given step.

        Parameters
        ----------
        stepName
            A String specifying the name of the step in which new surface vertex criteria
            assignments are to be defined.
        assignments
            A sequence of tuples specifying the surface vertex criteria assignments. Each tuple
            contains two entries:
            - A region or a material object or the SymbolicConstant GLOBAL specifying the surface to
            which the vertex criteria is assigned.
            - A Float or a SymbolicConstant specifying the vertex criteria value to be used for the
            surface. Possible values of the SymbolicConstant are ALL_VERTICES or NO_VERTICES.
        """
        ...

    @abaqus_method_doc
    def delete(self, indices: tuple):
        """The delete method allows you to delete existing surface vertex criteria assignments from
        a ContactStd object.

        Parameters
        ----------
        indices
            A sequence of Ints specifying the index of each surface vertex criteria assignment to
            delete.
        """
        ...



from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from ....UtilityAndView.abaqusConstants import *


@abaqus_class_doc
class FluidLeakoff:
    """The FluidLeakoff object specifies leak-off coefficients for pore pressure cohesive
    elements.

    .. note:: 
        This object can be accessed by::

            import material
            mdb.models[name].materials[name].fluidLeakoff
            import odbMaterial
            session.odbs[name].materials[name].fluidLeakoff

        The table data for this object are:
        
        The table data specify the following:

        - Fluid leak-off coefficient at top element surface.
        - Fluid leak-off coefficient at bottom element surface.
        - Temperature, if the data depend on temperature.
        - Value of the first field variable, if the data depend on field variables.
        - Value of the second field variable.
        - Etc.

        The corresponding analysis keywords are:

        - FLUID LEAKOFF
    """

    @abaqus_method_doc
    def __init__(
        self,
        temperatureDependency: Boolean = OFF,
        dependencies: int = 0,
        type: SymbolicConstant = COEFFICIENTS,
        table: tuple = (),
    ):
        """This method creates a FluidLeakoff object.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].materials[name].FluidLeakoff
                session.odbs[name].materials[name].FluidLeakoff

        Parameters
        ----------
        temperatureDependency
            A Boolean specifying whether the data depend on temperature. The default value is OFF.
        dependencies
            An Int specifying the number of field variable dependencies. The default value is 0.
        type
            A SymbolicConstant specifying the type of fluid leak-off. Possible values are
            COEFFICIENTS and USER. The default value is COEFFICIENTS.
        table
            A sequence of sequences of Floats specifying the items described below. The default
            value is an empty sequence.

        Returns
        -------
        FluidLeakoff
            A :py:class:`~abaqus.Material.Others.PoreFluidFlow.FluidLeakoff.FluidLeakoff` object.
        """
        ...

    @abaqus_method_doc
    def setValues(self, *args, **kwargs):
        """This method modifies the FluidLeakoff object."""
        ...



from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from ..UtilityAndView.abaqusConstants import *


@abaqus_class_doc
class OptimizationConstraint:
    """The OptimizationConstraint object constrains an optimization from making changes to the
    topology of the model.

    .. note:: 
        This object can be accessed by::

            import optimization
            mdb.models[name].optimizationTasks[name].optimizationConstraints[name]
    """

    #: A String specifying the optimization constraint repository key.
    name: str

    #: A String specifying the name of the design response to constrain.
    designResponse: str

    #: A Float specifying the value to which the design response should be constrained.
    restrictionValue: float

    #: A SymbolicConstant specifying the method used to constrain the design response. Possible
    #: values are ABSOLUTE_EQUAL, ABSOLUTE_GREATER_THAN_EQUAL, ABSOLUTE_LESS_THAN_EQUAL,
    #: RELATIVE_EQUAL, RELATIVE_GREATER_THAN_EQUAL, and RELATIVE_LESS_THAN_EQUAL. The default
    #: value is ABSOLUTE_EQUAL.
    restrictionMethod: SymbolicConstant = ABSOLUTE_EQUAL

    @abaqus_method_doc
    def __init__(
        self,
        name: str,
        designResponse: str,
        restrictionValue: float,
        restrictionMethod: SymbolicConstant = ABSOLUTE_EQUAL,
    ):
        """This method creates an OptimizationConstraint object.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].optimizationTasks[name].OptimizationConstraint

        Parameters
        ----------
        name
            A String specifying the optimization constraint repository key.
        designResponse
            A String specifying the name of the design response to constrain.
        restrictionValue
            A Float specifying the value to which the design response should be constrained.
        restrictionMethod
            A SymbolicConstant specifying the method used to constrain the design response. Possible
            values are ABSOLUTE_EQUAL, ABSOLUTE_GREATER_THAN_EQUAL, ABSOLUTE_LESS_THAN_EQUAL,
            RELATIVE_EQUAL, RELATIVE_GREATER_THAN_EQUAL, and RELATIVE_LESS_THAN_EQUAL. The default
            value is ABSOLUTE_EQUAL.

        Returns
        -------
        OptimizationConstraint
            An :py:class:`~abaqus.Optimization.OptimizationConstraint.OptimizationConstraint` object.

        Raises
        ------
        InvalidNameError
        RangeError
        """
        ...

    @abaqus_method_doc
    def setValues(self, restrictionMethod: SymbolicConstant = ABSOLUTE_EQUAL):
        """This method modifies the OptimizationConstraint object.

        Parameters
        ----------
        restrictionMethod
            A SymbolicConstant specifying the method used to constrain the design response. Possible
            values are ABSOLUTE_EQUAL, ABSOLUTE_GREATER_THAN_EQUAL, ABSOLUTE_LESS_THAN_EQUAL,
            RELATIVE_EQUAL, RELATIVE_GREATER_THAN_EQUAL, and RELATIVE_LESS_THAN_EQUAL. The default
            value is ABSOLUTE_EQUAL.

        Raises
        ------
        RangeError
        """
        ...



from abqpy.decorators import abaqus_class_doc
from .BoundaryConditionState import BoundaryConditionState
from ..UtilityAndView.abaqusConstants import *


@abaqus_class_doc
class ConnAccelerationBCState(BoundaryConditionState):
    """The ConnAccelerationBCState object stores the propagating data of a connector
    acceleration boundary condition in a step. One instance of this object is created
    internally by the ConnAccelerationBC object for each step. The instance is also deleted
    internally by the ConnAccelerationBC object.
    The ConnAccelerationBCState object has no constructor or methods.
    The ConnAccelerationBCState object is derived from the BoundaryConditionState object.

    .. note:: 
        This object can be accessed by::

            import load
            mdb.models[name].steps[name].boundaryConditionStates[name]

        The corresponding analysis keywords are:

        - CONNECTOR MOTION
    """

    #: A Float specifying the connector acceleration component in the connector's local
    #: 1-direction.
    a1: float = None

    #: A Float specifying the connector acceleration component in the connector's local
    #: 2-direction.
    a2: float = None

    #: A Float specifying the connector acceleration component in the connector's local
    #: 3-direction.
    a3: float = None

    #: A Float specifying the connector acceleration component in the connector's local
    #: 4-direction.
    ar1: float = None

    #: A Float specifying the connector acceleration component in the connector's local
    #: 5-direction.
    ar2: float = None

    #: A Float specifying the connector acceleration component in the connector's local
    #: 6-direction.
    ar3: float = None

    #: A SymbolicConstant specifying the propagation state of the connector acceleration
    #: component in the connector's local 1-direction. Possible values are UNSET, SET,
    #: UNCHANGED, FREED, and MODIFIED.
    a1State: SymbolicConstant = None

    #: A SymbolicConstant specifying the propagation state of the connector acceleration
    #: component in the connector's local 2-direction. Possible values are UNSET, SET,
    #: UNCHANGED, FREED, and MODIFIED.
    a2State: SymbolicConstant = None

    #: A SymbolicConstant specifying the propagation state of the connector acceleration
    #: component in the connector's local 3-direction. Possible values are UNSET, SET,
    #: UNCHANGED, FREED, and MODIFIED.
    a3State: SymbolicConstant = None

    #: A SymbolicConstant specifying the propagation state of the connector acceleration
    #: component in the connector's local 4-direction. Possible values are UNSET, SET,
    #: UNCHANGED, FREED, and MODIFIED.
    ar1State: SymbolicConstant = None

    #: A SymbolicConstant specifying the propagation state of the connector acceleration
    #: component in the connector's local 5-direction. Possible values are UNSET, SET,
    #: UNCHANGED, FREED, and MODIFIED.
    ar2State: SymbolicConstant = None

    #: A SymbolicConstant specifying the propagation state of the connector acceleration
    #: component in the connector's local 6-direction. Possible values are UNSET, SET,
    #: UNCHANGED, FREED, and MODIFIED.
    ar3State: SymbolicConstant = None

    #: A SymbolicConstant specifying the propagation state of the amplitude reference. Possible
    #: values are UNSET, SET, UNCHANGED, FREED, and MODIFIED.
    amplitudeState: SymbolicConstant = None

    #: A SymbolicConstant specifying the propagation state of the BoundaryConditionState object. Possible values are:
    #: 
    #: - NOT_YET_ACTIVE
    #: - CREATED
    #: - PROPAGATED
    #: - MODIFIED
    #: - DEACTIVATED
    #: - NO_LONGER_ACTIVE
    #: - TYPE_NOT_APPLICABLE
    #: - INSTANCE_NOT_APPLICABLE
    #: - PROPAGATED_FROM_BASE_STATE
    #: - MODIFIED_FROM_BASE_STATE
    #: - DEACTIVATED_FROM_BASE_STATE
    #: - BUILT_INTO_MODES
    status: SymbolicConstant = None

    #: A String specifying the name of the amplitude reference. The String is empty if the
    #: boundary condition has no amplitude reference.
    amplitude: str = ""



from abqpy.decorators import abaqus_class_doc


@abaqus_class_doc
class NodeQuery:
    """The NodeQuery object specifies nodes and their coordinates in a path. The NodeQuery
    object has no constructor or methods. Abaqus creates the **nodeQuery** member when you
    import the visualization module.

    .. note:: 
        This object can be accessed by::

            import visualization
            session.nodeQuery
    """

    #: An Int specifying the ID of the most recently queried node. If the last query was
    #: unsuccessful, **nodeID** = −1.
    nodeId: int = None

    #: A tuple of Floats specifying the **X**-, **Y**-, and **Z**-coordinates of the most recently
    #: queried node.
    nodePos: float = None



from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from .EventSeries import EventSeries
from .EventSeriesType import EventSeriesType
from ..Model.ModelBase import ModelBase
from ..UtilityAndView.abaqusConstants import *


@abaqus_class_doc
class TableCollectionModel(ModelBase):
    """Abaqus creates a Model object named `Model-1` when a session is started.

    .. note:: 
        This object can be accessed by::

            mdb.models[name]

    .. versionadded:: 2020
        The `TableCollectionModel` class was added.
    """

    @abaqus_method_doc
    def EventSeries(
        self,
        name: str,
        createStepName: str,
        eventSeriesType: str,
        transformType: str = NONE,
        timeSpan: str = STEP_TIME,
        transformations: str = "",
        fileName: str = "",
        data: str = "",
    ) -> EventSeries:
        """This method creates an EventSeries object.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].EventSeriesData

        Parameters
        ----------
        name
            A String specifying the repository key.
        createStepName
            A string specifying the step name.
        eventSeriesType
            A string specifying the type of event series.
        transformType
            A Symbolic constant specifying the type of transformation. Possible values are NONE,
            BOTH, TRANSLATE, and ROTATE. The default value is NONE.
        timeSpan
            A Symbolic constant specifying time. Possible values are TOTAL_TIME and STEP_TIME. The
            default value is STEP_TIME.
        transformations
            An Array specifying the required transformations over event series data.
        fileName
            A String specifying the filename.
        data
            An Array of double specifying the values of fields provided in EventSeriesType.

        Returns
        -------
        EventSeries
            An :py:class:`~abaqus.TableCollection.EventSeries.EventSeries` object.

        Raises
        ------
        RangeError
        """
        self.eventSeriesDatas[name] = eventSeries = EventSeries(
            name,
            createStepName,
            eventSeriesType,
            transformType,
            timeSpan,
            transformations,
            fileName,
            data,
        )
        return eventSeries

    @abaqus_method_doc
    def EventSeriesType(
        self, name: str, createStepName: str, fields: str = ""
    ) -> EventSeriesType:
        """This method creates an EventSeriesType object.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].EventSeriesType

        Parameters
        ----------
        name
            A String specifying the repository key.
        createStepName
            A string specifying the step name.
        fields
            A String array specifying fields. The default value is an empty array.

        Returns
        -------
        EventSeriesType
            A :py:class:`~abaqus.TableCollection.EventSeriesType.EventSeriesType` object.

        Raises
        ------
        RangeError
        """
        self.eventSeriesTypes[name] = eventSeriesType = EventSeriesType(
            name, createStepName, fields
        )
        return eventSeriesType



from abqpy.decorators import abaqus_class_doc


@abaqus_class_doc
class DataTable:
    """A DataTable is an object used to define the domain and data for a DiscreteField.

    .. note:: 
        This object can be accessed by::

            import field
            mdb.models[name].discreteFields[name].data[i]
    """

    #: An Int specifying the width of the data. Valid widths are 1, 6, 21, corresponding to
    #: scalar data, orientations and 4D tensors.
    dataWidth: int = None

    #: A String specifying the index.
    name: str = ""

    #: A String specifying the instance name.
    instanceName: str = ""

    #: A tuple of Ints specifying the domain node, element or integration point identifiers.
    domain: int = None

    #: A tuple of Floats specifying the data within the domain.
    table: float = None



from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from .ContactProperty import ContactProperty
from ..UtilityAndView.abaqusConstants import *


@abaqus_class_doc
class FluidCavityProperty(ContactProperty):
    """The FluidCavityProperty object is an interaction property that defines the fluid
    behavior for a surface-based fluid cavity.
    The FluidCavityProperty object is derived from the InteractionProperty object.

    .. note:: 
        This object can be accessed by::

            import interaction
            mdb.models[name].interactionProperties[name]

        The corresponding analysis keywords are:

        - FLUID BEHAVIOR
        - CAPACITY
        - FLUID BULK MODULUS
        - FLUID DENSITY
        - FLUID EXPANSION
        - MOLECULAR WEIGHT
    """

    #: A String specifying the interaction property repository key.
    name: str

    #: A SymbolicConstant specifying the type of fluid cavity property to be defined. Possible
    #: values are HYDRAULIC and PNEUMATIC. The default value is HYDRAULIC.
    definition: SymbolicConstant = HYDRAULIC

    #: None or a Float specifying the reference fluid density. This argument is applicable only
    #: when **definition** = HYDRAULIC, and is required in that case. The default value is None.
    fluidDensity: float = None

    #: None or a Float specifying the molecular weight of the ideal gas species. This argument
    #: is applicable only when **definition** = PNEUMATIC, and is required in that case. The
    #: default value is None.
    molecularWeight: float = None

    #: A Boolean specifying whether thermal expansion coefficients will be defined. This
    #: argument is applicable only when **definition** = HYDRAULIC. The default value is OFF.
    useExpansion: Boolean = OFF

    #: A Boolean specifying whether the thermal fluid expansion data will have temperature
    #: dependency. This argument is applicable only when **definition** = HYDRAULIC and when
    #: **useExpansion** = True. The default value is OFF.
    expansionTempDep: Boolean = OFF

    #: An Int specifying the number of field variable dependencies in the thermal fluid
    #: expansion data. This argument is applicable only when **definition** = HYDRAULIC and when
    #: **useExpansion** = True. The default value is 0.
    expansionDependencies: int = 0

    #: A Float specifying the reference temperature for the coefficient of thermal expansion.
    #: This argument is applicable only when **definition** = HYDRAULIC, when **useExpansion** = True,
    #: and when either **expansionTempDep** = True or when **expansionDependencies** is greater than
    #: 0. The default value is 0.0.
    referenceTemperature: float = 0

    #: A sequence of sequences of Floats specifying the thermal expansion coefficients. This
    #: argument is applicable only when **definition** = HYDRAULIC and when **useExpansion** = True.
    #: Each sequence contains the following data:
    #: 
    #: - The mean coefficient of thermal expansion.
    #: - Temperature, if the data depend on temperature.
    #: - Value of the first field variable, if the data depend on field variables.
    #: - Value of the second field variable.
    #: - Etc.
    expansionTable: tuple = ()

    #: A Boolean specifying whether fluid bulk modulus values will be defined. This argument is
    #: applicable only when **definition** = HYDRAULIC. The default value is OFF.
    useBulkModulus: Boolean = OFF

    #: A Boolean specifying whether the fluid bulk modulus data will have temperature
    #: dependency. This argument is applicable only when **definition** = HYDRAULIC and when
    #: **useBulkModulus** = True. The default value is OFF.
    bulkModulusTempDep: Boolean = OFF

    #: An Int specifying the number of field variable dependencies in the fluid bulk modulus
    #: data. This argument is applicable only when **definition** = HYDRAULIC and when
    #: **useBulkModulus** = True. The default value is 0.
    bulkModulusDependencies: int = 0

    #: A sequence of sequences of Floats specifying the fluid bulk modulus values. This
    #: argument is applicable only when **definition** = HYDRAULIC and when **useBulkModulus** = True.
    #: Each sequence contains the following data:
    #: 
    #: - The fluid bulk modulus.
    #: - Temperature, if the data depend on temperature.
    #: - Value of the first field variable, if the data depend on field variables.
    #: - Value of the second field variable.
    #: - Etc.
    bulkModulusTable: tuple = ()

    #: A Boolean specifying whether molar heat capacity values will be defined. This argument
    #: is applicable only when **definition** = PNEUMATIC. The default value is OFF.
    useCapacity: Boolean = OFF

    #: A SymbolicConstant specifying the method to define the molar heat capacity. Possible
    #: values are POLYNOMIAL and TABULAR. The default value is POLYNOMIAL.
    capacityType: SymbolicConstant = POLYNOMIAL

    #: A Boolean specifying whether the molar heat capacity data will have temperature
    #: dependency. This argument is applicable only when **definition** = PNEUMATIC, when
    #: **useCapacity** = True, and when **capacityType** = TABULAR. The default value is OFF.
    capacityTempDep: Boolean = OFF

    #: An Int specifying the number of field variable dependencies in the molar heat capacity
    #: data. This argument is applicable only when **definition** = PNEUMATIC, when
    #: **useCapacity** = True, and when **capacityType** = TABULAR. The default value is 0.
    capacityDependencies: int = 0

    #: A sequence of sequences of Floats specifying the molar heat capacity values in the form
    #: of a polynomial expression. This argument is applicable only when
    #: **definition** = PNEUMATIC, when **useCapacity** = True, and when **capacityType** = POLYNOMIAL. In
    #: this form, only one sequence is specified and that sequence contains the following data:
    #: 
    #: - The first molar heat capacity coefficient.
    #: - The second molar heat capacity coefficient.
    #: - The third molar heat capacity coefficient.
    #: - The fourth molar heat capacity coefficient.
    #: - The fifth molar heat capacity coefficient.
    #: 
    #: Alternatively, the sequence data may specify the molar heat capacity values at constant
    #: pressure for an ideal gas species. This argument is applicable only when
    #: **definition** = PNEUMATIC, when **useCapacity** = True, and when **capacityType** = TABULAR. Each
    #: sequence contains the following data:
    #: 
    #: - The molar heat capacity at constant pressure.
    #: - Temperature, if the data depend on temperature.
    #: - Value of the first field variable, if the data depend on field variables.
    #: - Value of the second field variable.
    #: - Etc.
    capacityTable: tuple = ()

    @abaqus_method_doc
    def __init__(
        self,
        name: str,
        definition: SymbolicConstant = HYDRAULIC,
        fluidDensity: float = None,
        molecularWeight: float = None,
        useExpansion: Boolean = OFF,
        expansionTempDep: Boolean = OFF,
        expansionDependencies: int = 0,
        referenceTemperature: float = 0,
        expansionTable: tuple = (),
        useBulkModulus: Boolean = OFF,
        bulkModulusTempDep: Boolean = OFF,
        bulkModulusDependencies: int = 0,
        bulkModulusTable: tuple = (),
        useCapacity: Boolean = OFF,
        capacityType: SymbolicConstant = POLYNOMIAL,
        capacityTempDep: Boolean = OFF,
        capacityDependencies: int = 0,
        capacityTable: tuple = (),
    ):
        """This method creates a FluidCavityProperty object.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].FluidCavityProperty

        Parameters
        ----------
        name
            A String specifying the interaction property repository key.
        definition
            A SymbolicConstant specifying the type of fluid cavity property to be defined. Possible
            values are HYDRAULIC and PNEUMATIC. The default value is HYDRAULIC.
        fluidDensity
            None or a Float specifying the reference fluid density. This argument is applicable only
            when **definition** = HYDRAULIC, and is required in that case. The default value is None.
        molecularWeight
            None or a Float specifying the molecular weight of the ideal gas species. This argument
            is applicable only when **definition** = PNEUMATIC, and is required in that case. The
            default value is None.
        useExpansion
            A Boolean specifying whether thermal expansion coefficients will be defined. This
            argument is applicable only when **definition** = HYDRAULIC. The default value is OFF.
        expansionTempDep
            A Boolean specifying whether the thermal fluid expansion data will have temperature
            dependency. This argument is applicable only when **definition** = HYDRAULIC and when
            **useExpansion** = True. The default value is OFF.
        expansionDependencies
            An Int specifying the number of field variable dependencies in the thermal fluid
            expansion data. This argument is applicable only when **definition** = HYDRAULIC and when
            **useExpansion** = True. The default value is 0.
        referenceTemperature
            A Float specifying the reference temperature for the coefficient of thermal expansion.
            This argument is applicable only when **definition** = HYDRAULIC, when **useExpansion** = True,
            and when either **expansionTempDep** = True or when **expansionDependencies** is greater than
            0. The default value is 0.0.
        expansionTable
            A sequence of sequences of Floats specifying the thermal expansion coefficients. This
            argument is applicable only when **definition** = HYDRAULIC and when **useExpansion** = True.
            Each sequence contains the following data:
            
            - The mean coefficient of thermal expansion.
            - Temperature, if the data depend on temperature.
            - Value of the first field variable, if the data depend on field variables.
            - Value of the second field variable.
            - Etc.
        useBulkModulus
            A Boolean specifying whether fluid bulk modulus values will be defined. This argument is
            applicable only when **definition** = HYDRAULIC. The default value is OFF.
        bulkModulusTempDep
            A Boolean specifying whether the fluid bulk modulus data will have temperature
            dependency. This argument is applicable only when **definition** = HYDRAULIC and when
            **useBulkModulus** = True. The default value is OFF.
        bulkModulusDependencies
            An Int specifying the number of field variable dependencies in the fluid bulk modulus
            data. This argument is applicable only when **definition** = HYDRAULIC and when
            **useBulkModulus** = True. The default value is 0.
        bulkModulusTable
            A sequence of sequences of Floats specifying the fluid bulk modulus values. This
            argument is applicable only when **definition** = HYDRAULIC and when **useBulkModulus** = True.
            Each sequence contains the following data:
            
            - The fluid bulk modulus.
            - Temperature, if the data depend on temperature.
            - Value of the first field variable, if the data depend on field variables.
            - Value of the second field variable.
            - Etc.
        useCapacity
            A Boolean specifying whether molar heat capacity values will be defined. This argument
            is applicable only when **definition** = PNEUMATIC. The default value is OFF.
        capacityType
            A SymbolicConstant specifying the method to define the molar heat capacity. Possible
            values are POLYNOMIAL and TABULAR. The default value is POLYNOMIAL.
        capacityTempDep
            A Boolean specifying whether the molar heat capacity data will have temperature
            dependency. This argument is applicable only when **definition** = PNEUMATIC, when
            **useCapacity** = True, and when **capacityType** = TABULAR. The default value is OFF.
        capacityDependencies
            An Int specifying the number of field variable dependencies in the molar heat capacity
            data. This argument is applicable only when **definition** = PNEUMATIC, when
            **useCapacity** = True, and when **capacityType** = TABULAR. The default value is 0.
        capacityTable
            A sequence of sequences of Floats specifying the molar heat capacity values in the form
            of a polynomial expression. This argument is applicable only when
            **definition** = PNEUMATIC, when **useCapacity** = True, and when **capacityType** = POLYNOMIAL. In
            this form, only one sequence is specified and that sequence contains the following data:
            
            - The first molar heat capacity coefficient.
            - The second molar heat capacity coefficient.
            - The third molar heat capacity coefficient.
            - The fourth molar heat capacity coefficient.
            - The fifth molar heat capacity coefficient.
              
            Alternatively, the sequence data may specify the molar heat capacity values at constant
            pressure for an ideal gas species. This argument is applicable only when
            **definition** = PNEUMATIC, when **useCapacity** = True, and when **capacityType** = TABULAR. Each
            sequence contains the following data:
            
            - The molar heat capacity at constant pressure.
            - Temperature, if the data depend on temperature.
            - Value of the first field variable, if the data depend on field variables.
            - Value of the second field variable.
            - Etc.

        Returns
        -------
        FluidCavityProperty
            A :py:class:`~abaqus.Interaction.FluidCavityProperty.FluidCavityProperty` object.
        """
        super().__init__(name)

    @abaqus_method_doc
    def setValues(
        self,
        definition: SymbolicConstant = HYDRAULIC,
        fluidDensity: float = None,
        molecularWeight: float = None,
        useExpansion: Boolean = OFF,
        expansionTempDep: Boolean = OFF,
        expansionDependencies: int = 0,
        referenceTemperature: float = 0,
        expansionTable: tuple = (),
        useBulkModulus: Boolean = OFF,
        bulkModulusTempDep: Boolean = OFF,
        bulkModulusDependencies: int = 0,
        bulkModulusTable: tuple = (),
        useCapacity: Boolean = OFF,
        capacityType: SymbolicConstant = POLYNOMIAL,
        capacityTempDep: Boolean = OFF,
        capacityDependencies: int = 0,
        capacityTable: tuple = (),
    ):
        """This method modifies the FluidCavityProperty object.

        Parameters
        ----------
        definition
            A SymbolicConstant specifying the type of fluid cavity property to be defined. Possible
            values are HYDRAULIC and PNEUMATIC. The default value is HYDRAULIC.
        fluidDensity
            None or a Float specifying the reference fluid density. This argument is applicable only
            when **definition** = HYDRAULIC, and is required in that case. The default value is None.
        molecularWeight
            None or a Float specifying the molecular weight of the ideal gas species. This argument
            is applicable only when **definition** = PNEUMATIC, and is required in that case. The
            default value is None.
        useExpansion
            A Boolean specifying whether thermal expansion coefficients will be defined. This
            argument is applicable only when **definition** = HYDRAULIC. The default value is OFF.
        expansionTempDep
            A Boolean specifying whether the thermal fluid expansion data will have temperature
            dependency. This argument is applicable only when **definition** = HYDRAULIC and when
            **useExpansion** = True. The default value is OFF.
        expansionDependencies
            An Int specifying the number of field variable dependencies in the thermal fluid
            expansion data. This argument is applicable only when **definition** = HYDRAULIC and when
            **useExpansion** = True. The default value is 0.
        referenceTemperature
            A Float specifying the reference temperature for the coefficient of thermal expansion.
            This argument is applicable only when **definition** = HYDRAULIC, when **useExpansion** = True,
            and when either **expansionTempDep** = True or when **expansionDependencies** is greater than
            0. The default value is 0.0.
        expansionTable
            A sequence of sequences of Floats specifying the thermal expansion coefficients. This
            argument is applicable only when **definition** = HYDRAULIC and when **useExpansion** = True.
            Each sequence contains the following data:
            
            - The mean coefficient of thermal expansion.
            - Temperature, if the data depend on temperature.
            - Value of the first field variable, if the data depend on field variables.
            - Value of the second field variable.
            - Etc.
        useBulkModulus
            A Boolean specifying whether fluid bulk modulus values will be defined. This argument is
            applicable only when **definition** = HYDRAULIC. The default value is OFF.
        bulkModulusTempDep
            A Boolean specifying whether the fluid bulk modulus data will have temperature
            dependency. This argument is applicable only when **definition** = HYDRAULIC and when
            **useBulkModulus** = True. The default value is OFF.
        bulkModulusDependencies
            An Int specifying the number of field variable dependencies in the fluid bulk modulus
            data. This argument is applicable only when **definition** = HYDRAULIC and when
            **useBulkModulus** = True. The default value is 0.
        bulkModulusTable
            A sequence of sequences of Floats specifying the fluid bulk modulus values. This
            argument is applicable only when **definition** = HYDRAULIC and when **useBulkModulus** = True.
            Each sequence contains the following data:
            
            - The fluid bulk modulus.
            - Temperature, if the data depend on temperature.
            - Value of the first field variable, if the data depend on field variables.
            - Value of the second field variable.
            - Etc.
        useCapacity
            A Boolean specifying whether molar heat capacity values will be defined. This argument
            is applicable only when **definition** = PNEUMATIC. The default value is OFF.
        capacityType
            A SymbolicConstant specifying the method to define the molar heat capacity. Possible
            values are POLYNOMIAL and TABULAR. The default value is POLYNOMIAL.
        capacityTempDep
            A Boolean specifying whether the molar heat capacity data will have temperature
            dependency. This argument is applicable only when **definition** = PNEUMATIC, when
            **useCapacity** = True, and when **capacityType** = TABULAR. The default value is OFF.
        capacityDependencies
            An Int specifying the number of field variable dependencies in the molar heat capacity
            data. This argument is applicable only when **definition** = PNEUMATIC, when
            **useCapacity** = True, and when **capacityType** = TABULAR. The default value is 0.
        capacityTable
            A sequence of sequences of Floats specifying the molar heat capacity values in the form
            of a polynomial expression. This argument is applicable only when
            **definition** = PNEUMATIC, when **useCapacity** = True, and when **capacityType** = POLYNOMIAL. In
            this form, only one sequence is specified and that sequence contains the following data:
            
            - The first molar heat capacity coefficient.
            - The second molar heat capacity coefficient.
            - The third molar heat capacity coefficient.
            - The fourth molar heat capacity coefficient.
            - The fifth molar heat capacity coefficient.
            
            Alternatively, the sequence data may specify the molar heat capacity values at constant
            pressure for an ideal gas species. This argument is applicable only when
            **definition** = PNEUMATIC, when **useCapacity** = True, and when **capacityType** = TABULAR. Each
            sequence contains the following data:
              
            - The molar heat capacity at constant pressure.
            - Temperature, if the data depend on temperature.
            - Value of the first field variable, if the data depend on field variables.
            - Value of the second field variable.
            - Etc.
        """
        ...



from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from .ContactProperty import ContactProperty


@abaqus_class_doc
class FluidInflatorProperty(ContactProperty):
    """The FluidInflatorProperty object is an interaction property that defines a fluid
    inflator property to model the deployment of an airbag. The inflator property defines
    the mass flow rate and temperature as a function of inflation time either directly or by
    entering tank test data. It also defines the mixture of gases entering the fluid cavity.
    The FluidInflatorProperty object is derived from the InteractionProperty object.

    .. note::
        This object can be accessed by::

            import interaction
            mdb.models[name].interactionProperties[name]

        The table data for this object are:

        - If **definition** = DUAL PRESSURE, the table data specify the following:

            - Inflation time.
            - Inflator pressure.
            - Tank pressure.

        - If **definition** = PRESSURE AND MASS, the table data specify the following:

            - Inflation time.
            - Inflator pressure.
            - Inflator mass flow rate.

        - If **definition** = TANK TEST, the table data specify the following:

            - Inflation time.
            - Inflator gas temperature.
            - Tank pressure.

        - If **definition** = TEMPERATURE AND MASS, the table data specify the following:

            - Inflation time.
            - Inflator gas temperature.
            - Inflator mass flow rate.

        The corresponding analysis keywords are:

        - FLUID INFLATOR PROPERTY

    .. versionadded:: 2019
        The `FluidInflatorProperty` class was added.
    """

    @abaqus_method_doc
    def __init__(
        self,
        name: str,
        definition: str,
        effectiveArea: float,
        tankVolume: float,
        dischargeCoefficient: float = None,
        dataTable: tuple = (),
        numFluids: int = None,
        mixtureType: str = "",
        inflationTime: tuple = (),
        fluidbehaviorName: tuple = (),
        massFraction: tuple = (),
    ):
        """This method creates a FluidInflatorProperty object.

        .. note::
            This function can be accessed by::

                mdb.models[name].FluidInflatorProperty

        Parameters
        ----------
        name
            A String specifying the interaction property repository key.
        definition
            A Symbolic constant specifying the method used for modeling the flow characteristics of
            inflators. The default value is **definition** = DUAL PRESSURE. The possible values are DUAL
            PRESSURE, PRESSURE AND MASS, TANK TEST, and TEMPERATURE AND MASS.
        effectiveArea
            A Float specifying the total inflator orifice area. This argument is applicable only if
            **definition** = DUAL PRESSURE or **definition** = PRESSURE AND MASS.
        tankVolume
            A Float specifying the tank volume. This argument is applicable only if
            **definition** = DUAL PRESSURE or **definition** = TANK TEST.
        dischargeCoefficient
            A Float specifying the discharge coefficient. This argument is applicable only if
            **definition** = DUAL PRESSURE or **definition** = PRESSURE AND MASS.
        dataTable
            A sequence of sequences of Floats specifying the items described in the "Table data"
            section below.
        numFluids
            An Int specifying the number of gas species used for this inflator.
        mixtureType
            A Symbolic constant specifying whether to use mass fraction or the molar fraction for a
            mixture of ideal gases. The default value is MASS FRACTION. The possible values are MASS
            FRACTION or MOLAR FRACTION.
        inflationTime
            A sequence of sequences of Floats specifying the inflation time.
        fluidbehaviorName
            A sequence of sequences of Strings specifying fluid behavior names.
        massFraction
            A sequence of sequences of Floats specifying the mass fraction or the molar fraction
            corresponding to entered fluid behavior.

        Returns
        -------
            A FluidInflatorProperty object.
        """
        super().__init__(name)

    @abaqus_method_doc
    def setValues(
        self,
        dischargeCoefficient: float = None,
        dataTable: tuple = (),
        numFluids: int = None,
        mixtureType: str = "",
        inflationTime: tuple = (),
        fluidbehaviorName: tuple = (),
        massFraction: tuple = (),
    ):
        """This method modifies the FluidInflatorProperty object.

        Parameters
        ----------
        dischargeCoefficient
            A Float specifying the discharge coefficient. This argument is applicable only if
            **definition** = DUAL PRESSURE or **definition** = PRESSURE AND MASS.
        dataTable
            A sequence of sequences of Floats specifying the items described in the "Table data"
            section below.
        numFluids
            An Int specifying the number of gas species used for this inflator.
        mixtureType
            A Symbolic constant specifying whether to use mass fraction or the molar fraction for a
            mixture of ideal gases. The default value is MASS FRACTION. The possible values are MASS
            FRACTION or MOLAR FRACTION.
        inflationTime
            A sequence of sequences of Floats specifying the inflation time.
        fluidbehaviorName
            A sequence of sequences of Strings specifying fluid behavior names.
        massFraction
            A sequence of sequences of Floats specifying the mass fraction or the molar fraction
            corresponding to entered fluid behavior.
        """
        ...



import typing

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from ..Assembly.AssemblyBase import AssemblyBase
from ..Datum.DatumCsys import DatumCsys
from ..Mesh.MeshElement import MeshElement
from ..Mesh.MeshNode import MeshNode
from ..Region.Region import Region
from ..UtilityAndView.abaqusConstants import *


@abaqus_class_doc
class MeshEditAssembly(AssemblyBase):
    """An :py:class:`~abaqus.Assembly.Assembly.Assembly` object is a container for instances of parts. The Assembly object has no
    constructor command. Abaqus creates the **rootAssembly** member when a Model object is
    created.

    .. note:: 
        This object can be accessed by::

            import assembly
            mdb.models[name].rootAssembly
    """

    @abaqus_method_doc
    def collapseMeshEdge(self, edge: str, collapseMethod: SymbolicConstant):
        """This method collapses an edge of a quadrilateral or triangular element of a part
        instance.

        Parameters
        ----------
        edge
            A single MeshEdge object specifying the element edge to collapse.
        collapseMethod
            A SymbolicConstant specifying the method used to collapse the edge. Possible values are
            FORWARD, REVERSE, and AVERAGE.
        """
        ...

    @abaqus_method_doc
    def combineElement(self, elements: tuple):
        """This method combines two triangular elements of a part instance.

        Parameters
        ----------
        elements
            A sequence of triangular MeshElement objects specifying the elements to combine.
        """
        ...

    @abaqus_method_doc
    def deleteElement(
        self, elements: typing.Tuple[MeshElement, ...], deleteUnreferencedNodes: Boolean = OFF
    ):
        """This method deletes the given elements from a part instance. The elements must have been
        generated using the bottom-up meshing technique.

        Parameters
        ----------
        elements
            A sequence of MeshElement objects or a Set object containing elements.
        deleteUnreferencedNodes
            A Boolean specifying whether to delete all those associated nodes that become
            unreferenced after the given elements are deleted. The default value is OFF.
        """
        ...

    @abaqus_method_doc
    def projectNode(self, nodes: typing.Tuple[MeshNode, ...], projectionReference: str):
        """This method projects the given nodes of a part instance onto a mesh entity, geometric
        entity, or a datum object.

        Parameters
        ----------
        nodes
            A sequence of MeshNode objects to be projected.
        projectionReference
            An object specifying the target for the node projection operation. The
            **projectionReference** can be any one of the following objects: MeshNode, MeshEdge,
            MeshFace, ConstrainedSketchVertex, Edge, Face, DatumPoint, DatumAxis, or DatumPlane.
        """
        ...

    @abaqus_method_doc
    def editNode(
        self,
        nodes: typing.Tuple[MeshNode, ...],
        coordinate1: float = None,
        coordinate2: float = None,
        coordinate3: float = None,
        coordinates: tuple = (),
        offset1: float = None,
        offset2: float = None,
        offset3: float = None,
        localCsys: DatumCsys = None,
        projectToGeometry: Boolean = ON,
    ):
        """This method changes the coordinates of the given nodes on a part instance.

        Parameters
        ----------
        nodes
            A sequence of MeshNode objects or a Set object containing nodes.
        coordinate1
            A Float specifying the value of the first coordinate. If **coordinate1** and **offset1** are
            unspecified, the existing value does not change.
        coordinate2
            A Float specifying the value of the second coordinate. If **coordinate2** and **offset2**
            are unspecified, the existing value does not change.
        coordinate3
            A Float specifying the value of the third coordinate. If **coordinate3** and **offset3** are
            unspecified, the existing value does not change.
        coordinates
            A sequence of three-dimensional coordinate tuples specifying the coordinates for each of
            the given nodes. When specified, the number of coordinate tuples must match the number
            of given nodes, and be ordered to correspond to the given nodes in *ascending order*
            according to index. Furthermore, **coordinate1**, **coordinate2**, **coordinate3**, **offset1**,
            **offset2**, or **offset3** may not be specified.
        offset1
            A Float specifying an offset to apply to the value of the first coordinate of the
            specified nodes.
        offset2
            A Float specifying an offset to apply to the value of the second coordinate of the
            specified nodes.
        offset3
            A Float specifying an offset to apply to the value of the third coordinate of the
            specified nodes.
        localCsys
            A :py:class:`~abaqus.Datum.DatumCsys.DatumCsys` object specifying the local coordinate system. If unspecified, the global
            coordinate system will be used.
        projectToGeometry
            A Boolean specifying whether to project nodes back to their original geometry. For
            example, if a node is on a face, this method first positions the node at the new
            location and then projects it back to the original face. The default value is ON.

        Raises
        ------
        A coordinate and an offset may not both be specified for the same coordinate component
        """
        ...

    @typing.overload
    @abaqus_method_doc
    def mergeNodes(
        self,
        nodes: typing.Tuple[MeshNode, ...],
        tolerance: float = None,
        removeDuplicateElements: Boolean = True,
    ):
        """Merge the nodes of a part instance. The nodes must have been generated using the
        bottom-up meshing technique.

        Parameters
        ----------
        nodes
            A sequence of MeshNode objects specifying the nodes to merge.
        tolerance
            A Float specifying the maximum distance between nodes that will be merged to a single
            node. The location of the new node is the average position of the merged nodes. The
            default value is 10-6.
        removeDuplicateElements
            A Boolean specifying whether elements with the same connectivity after the merge will
            merged into a single element. The default value is True.
        """
        ...

    @typing.overload
    @abaqus_method_doc
    def mergeNodes(
        self, node1: MeshNode, node2: MeshNode, removeDuplicateElements: Boolean = True
    ):
        """Merge two nodes of a part instance. At least one of the two nodes must have been
        generated using the bottom-up meshing technique.

        Parameters
        ----------
        node1
            A :py:class:`~abaqus.Mesh.MeshNode.MeshNode` object specifying the first node to merge.
        node2
            A :py:class:`~abaqus.Mesh.MeshNode.MeshNode` object specifying the second node to merge.
        removeDuplicateElements
            A Boolean specifying whether elements with the same connectivity after the merge will
            merged into a single element. The default value is True.
        """
        ...

    @abaqus_method_doc
    def mergeNodes(self, *args, **kwargs):
        ...

    def splitElement(self, elements: tuple):
        """This method splits quadrilateral elements into triangular elements.

        Parameters
        ----------
        elements
            A sequence of quadrilateral MeshElement objects specifying the elements to split. Each
            quadrilateral element is split into two triangular elements by the shorter diagonal.
        """
        ...

    @abaqus_method_doc
    def splitMeshEdge(self, edge: str, parameter: float = 0):
        """This method splits an edge of a quadrilateral or triangular element of a part instance.

        Parameters
        ----------
        edge
            A single MeshEdge object specifying the element edge to split.
        parameter
            A Float specifying the normalized distance along the **edge** at which to split. Possible
            values are 0.0 << **parameter** << 1.0. The default value is 0.5.
        """
        ...

    @abaqus_method_doc
    def swapMeshEdge(self, edge: str):
        """This method swaps the diagonal of two adjacent triangular elements of a part instance.

        Parameters
        ----------
        edge
            A single MeshEdge object specifying the element edge to swap.
        """
        ...

    @abaqus_method_doc
    def generateMeshByOffset(
        self,
        region: Region,
        meshType: str,
        totalThickness: float,
        distanceBetweenLayers: float,
        numLayers: int,
        offsetDirection: str = OUTWARD,
        initialOffset: float = 0.0,
        shareNodes: str = False,
        deleteBaseElements: Boolean = False,
        constantThicknessCorners: Boolean = False,
        extendElementSets: Boolean = False,
    ):
        """This method generates a solid or shell mesh from an orphan mesh surface by generating
        layers of elements that propagate out normal to the surface boundary.

        Parameters
        ----------
        region
            A :py:class:`~abaqus.Region.Region.Region` object specifying the domain to be offset.
        meshType
            A Symbolic Constant specifying the type of mesh to be generated. Possible values are
            SOLID or SHELL.
        totalThickness
            A Float specifying the total thickness of the solid layers. This argument applies only
            when **meshType** = SOLID.
        distanceBetweenLayers
            A Float specifying the distance between shell layers. This argument applies only when
            **meshType** = SHELL.
        numLayers
            An Int specifying the number of element layers to be generated.
        offsetDirection
            A Symbolic Constant specifying the direction of the offset. This argument is required
            only when the given region relates to a shell mesh. Possible values are OUTWARD, INWARD,
            and BOTH. The default value is OUTWARD.
        initialOffset
            A Float specifying the magnitude of the initial offset. The default value is zero.
        shareNodes
            Boolean specifying whether the first layer of nodes should be shared with nodes on the
            base surface. The default value is False.
        deleteBaseElements
            A Boolean specifying whether to delete the shell elements after the offset layers are
            generated. The default value is False. This argument applies only when **meshType** = SHELL.
        constantThicknessCorners
            A Boolean specifying whether to use element-based thickness or nodal-based thickness.
            The default value is False.
        extendElementSets
            A Boolean specifying whether existing element sets that include base elements will be
            extended to include corresponding offset elements. The default value is False.
        """
        ...

    @abaqus_method_doc
    def redoMeshEdit(self):
        """This method executes the edit mesh or the bottom-up meshing operation most recently
        undone by the undoMeshEdit method on an assembly. A redo action must be currently
        available for the assembly. This implies that the user must have executed the
        undoMeshEdit method on the assembly and that the user has not subsequently executed any
        further edit mesh commands on the assembly. It also implies that the user provided a
        sufficient cache allowance to store the undo operation.
        """
        ...

    @abaqus_method_doc
    def undoMeshEdit(self):
        """This method undoes the most recent edit mesh or the bottom-up meshing operation on an
        assembly and restores the mesh on the affected part instance to its previous state. An
        edit mesh undo action must be available for the assembly. This implies that prior to
        executing an edit mesh command on the assembly, the user enabled edit mesh undo with a
        sufficient cache allowance to store the edit mesh operation.
        """
        ...



import typing

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from ..TableCollection.DataTable import DataTable
from ..TableCollection.ParameterTable import ParameterTable
from ..TableCollection.PropertyTable import PropertyTable


@abaqus_class_doc
class TableCollection:
    """A TableCollection is an object used to define the containers that encapsulate the
    ParameterTable and PropertyTable objects.

    .. note:: 
        This object can be accessed by::

            mdb.models[name].tableCollections[name]

        The corresponding analysis keywords are:

        - TABLE COLLECTION

    .. versionadded:: 2020
        The `TableCollection` class was added.
    """

    #: A repository of the PropertyTable object.
    propertyTables: typing.Dict[str, PropertyTable] = {}

    #: A repository of the ParameterTable object
    parameterTables: typing.Dict[str, ParameterTable] = {}

    #: sequence of the DataTable object
    dataTables: typing.List[DataTable] = []

    @abaqus_method_doc
    def __init__(self, name: str):
        """This method creates a TableCollection object and places it in the tableCollections
        repository.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].TableCollection

        Parameters
        ----------
        name
            A String specifying the repository key.

        Returns
        -------
        TableCollection
            A :py:class:`~abaqus.TableCollection.TableCollection.TableCollection` object.
        """
        ...

    @abaqus_method_doc
    def DataTable(self, label: str) -> DataTable:
        """This method creates a DataTable object and places it in the dataTables array.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].TableCollection

        Parameters
        ----------
        label
            A String specifying a unique label name for the current ParameterTable object.

        Returns
        -------
        DataTable
            A :py:class:`~abaqus.Field.DataTable.DataTable` object.

        Raises
        ------
        AbaqusException
        """
        dataTable = DataTable(label)
        self.dataTables.append(dataTable)
        return dataTable

    @abaqus_method_doc
    def ParameterTable(self, name: str) -> ParameterTable:
        """This method creates a ParameterTable object and places it in the parameterTables
        repository.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].TableCollection

        Parameters
        ----------
        name
            A String specifying the repository key.

        Returns
        -------
        ParameterTable
            A :py:class:`~abaqus.TableCollection.ParameterTable.ParameterTable` object.
        """
        self.parameterTables[name] = parameterTable = ParameterTable(name)
        return parameterTable

    @abaqus_method_doc
    def PropertyTable(
        self, name: str, properties: str, variables: str = ""
    ) -> PropertyTable:
        """This method creates a PropertyTable object.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].TableCollection

        Parameters
        ----------
        name
            A String specifying the repository key.
        properties
            A string array specifying the multiple properties to build the parameter table type.
        variables
            A String array specifying multiple independent variables. The default value is an empty
            array.

        Returns
        -------
        PropertyTable
            A :py:class:`~abaqus.TableCollection.PropertyTable.PropertyTable` object.

        Raises
        ------
        RangeError
        """
        self.propertyTables[name] = propertyTable = PropertyTable(
            name, properties, variables
        )
        return propertyTable



import typing

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from ..UtilityAndView.abaqusConstants import *


@abaqus_class_doc
class FractureCriterion:
    r"""The FractureCriterion object specifies fractureCriterion options for a contact
    interaction property.

    .. note:: 
        This object can be accessed by::

            import interaction
            mdb.models[name].interactionProperties[name].fractureCriterion

        The table data for this object are:
        
        Table data for **initTable**:
        
        If **type** = VCCT for **mixedModeBehavior** = BK or REEDER, the table data specify the following:
        
        - Mode I critical energy release rate, :math:`G_{I C}`.
        - Mode II critical energy release rate, :math:`G_{I I C}`.
        - Mode III critical energy release rate, :math:`G_{I I I C}`.
        - Exponent, :math:`\eta`.
        - Temperature, if the data depend on temperature.
        - Value of the first field variable, if the data depend on field variables.
        - Value of the second field variable.
        - Etc.
        
        If **type** = VCCT for **mixedModeBehavior** = POWER, the table data specify the following:
        
        - Mode I critical energy release rate, :math:`G_{I C}`.
        - Mode II critical energy release rate, :math:`G_{I I C}`.
        - Mode III critical energy release rate, :math:`G_{I I I C}`.
        - Exponent, :math:`am`.
        - Exponent, :math:`an`.
        - Exponent, :math:`ao`.
        - Value of the first field variable, if the data depend on field variables.
        - Value of the second field variable.
        - Etc.
        
        If **type** = ENHANCED VCCT for **mixedModeBehavior** = BK or REEDER, the table data specify the following:
        
        - Mode I critical energy release rate for onset crack, :math:`G_{I C}`.
        - Mode II critical energy release rate for onset crack, :math:`G_{I I C}`.
        - Mode III critical energy release rate for onset crack, :math:`G_{I I I C}`.
        - Mode I critical energy release rate for crack propagation, :math:`G_{I C}`.
        - Mode II critical energy release rate for crack propagation, :math:`G_{I I C}`.
        - Mode III critical energy release rate for crack propagation, :math:`G_{I I I C}`.
        - Exponent, :math:`\eta`
        - Temperature, if the data depend on temperature.
        - Value of the first field variable, if the data depend on field variables.
        - Value of the second field variable.
        - Etc.
        
        If **type** = ENHANCED VCCT for **mixedModeBehavior** = POWER, the table data specify the following:
        
        - Mode I critical energy release rate for onset crack, :math:`G_{I C}`.
        - Mode II critical energy release rate for onset crack, :math:`G_{I I C}`.
        - Mode III critical energy release rate for onset crack, :math:`G_{I I I C}`.
        - Mode I critical energy release rate for crack propagation, :math:`G_{I C}`.
        - Mode II critical energy release rate for crack propagation, :math:`G_{I I C}`.
        - Mode III critical energy release rate for crack propagation, :math:`G_{I I I C}`.
        - Exponent, :math:`am`.
        - Exponent, :math:`an`.
        - Exponent, :math:`ao`.
        - Value of the first field variable, if the data depend on field variables.
        - Value of the second field variable.
        - Etc.

        The corresponding analysis keywords are:

        - FRACTURE CRITERION
    """

    #: A sequence of sequences of Floats specifying the value defining the fracture criterion.
    #: The items in the table data are described below.
    initTable: tuple

    #: A SymbolicConstant specifying the type of data used to define the fracture criterion.
    #: Possible values are VCCT and ENHANCED VCCT. The default value is VCCT.
    type: SymbolicConstant = VCCT

    #: A SymbolicConstant specifying the mixed mode behavior type used to define fracture
    #: criterion. Possible values are BK, POWER, and REEDER. The default value is BK.
    mixedModeBehavior: SymbolicConstant = BK

    #: A Boolean specifying whether the fracture criterion data depend on temperature. The
    #: default value is OFF.
    temperatureDependency: Boolean = OFF

    #: An Int specifying the number of fracture criterion data field variables. The default
    #: value is 0.
    dependencies: int = 0

    #: A Float specifying the tolerance for VCCT\Enhanced VCCT type. The default value is 0.2.
    tolerance: float = 0

    #: A SymbolicConstant specifying whether to include unstable crack growth tolerance in
    #: fracture criterion. Possible values are ON and OFF. The default value is OFF.
    specifyUnstableCrackProp: SymbolicConstant = OFF

    #: The SymbolicConstant DEFAULT or a Float specifying the tolerance for unstable crack
    #: propagation. This parameter specified only if **specifyUnstableCrackProp** = ON. The default
    #: value is DEFAULT.
    unstableTolerance: typing.Union[SymbolicConstant, float] = DEFAULT

    @abaqus_method_doc
    def __init__(
        self,
        initTable: tuple,
        type: SymbolicConstant = VCCT,
        mixedModeBehavior: SymbolicConstant = BK,
        temperatureDependency: Boolean = OFF,
        dependencies: int = 0,
        tolerance: float = 0,
        specifyUnstableCrackProp: SymbolicConstant = OFF,
        unstableTolerance: typing.Union[SymbolicConstant, float] = DEFAULT,
    ):
        r"""This method creates a FractureCriterion object.

        .. note:: 
            This function can be accessed by::

                mdb.models[name].interactionProperties[name].FractureCriterion

        Parameters
        ----------
        initTable
            A sequence of sequences of Floats specifying the value defining the fracture criterion.
            The items in the table data are described below.
        type
            A SymbolicConstant specifying the type of data used to define the fracture criterion.
            Possible values are VCCT and ENHANCED VCCT. The default value is VCCT.
        mixedModeBehavior
            A SymbolicConstant specifying the mixed mode behavior type used to define fracture
            criterion. Possible values are BK, POWER, and REEDER. The default value is BK.
        temperatureDependency
            A Boolean specifying whether the fracture criterion data depend on temperature. The
            default value is OFF.
        dependencies
            An Int specifying the number of fracture criterion data field variables. The default
            value is 0.
        tolerance
            A Float specifying the tolerance for VCCT\Enhanced VCCT type. The default value is 0.2.
        specifyUnstableCrackProp
            A SymbolicConstant specifying whether to include unstable crack growth tolerance in
            fracture criterion. Possible values are ON and OFF. The default value is OFF.
        unstableTolerance
            The SymbolicConstant DEFAULT or a Float specifying the tolerance for unstable crack
            propagation. This parameter specified only if **specifyUnstableCrackProp** = ON. The default
            value is DEFAULT.

        Returns
        -------
        FractureCriterion
            A :py:class:`~abaqus.Interaction.FractureCriterion.FractureCriterion` object.
        """
        ...

    @abaqus_method_doc
    def setValues(self, *args, **kwargs):
        """This method modifies the FractureCriterion object."""
        ...



from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from ..UtilityAndView.abaqusConstants import *


@abaqus_class_doc
class OdbDataFrame:
    """The OdbDataFrame object.

    .. note:: 
        This object can be accessed by::

            import visualization
            session.odbData[name].steps[i].frames[i]
    """

    @abaqus_method_doc
    def setValues(self, activateFrame: Boolean, update: Boolean = OFF):
        """This method modifies the OdbDataFrame object.

        Parameters
        ----------
        activateFrame
            A Boolean specifying whether to activate the frame.
        update
            A Boolean specifying whether to update the model. The default value is ON
        """
        # TODO: Implement this method
        ...



import typing

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from ..UtilityAndView.abaqusConstants import *
from .._OptionsBase import _OptionsBase


@abaqus_class_doc
class PageSetupOptions(_OptionsBase):
    """The PageSetupOptions object stores the settings that Abaqus uses when printing using a
    Windows printer. The PageSetupOptions object has no constructor. Abaqus creates the
    **pageSetupOptions** member when a session is started.

    .. note:: 
        This object can be accessed by::

            session.pageSetupOptions
    """
    #: The SymbolicConstant SIZE_ON_SCREEN or a pair of Floats specifying the width and height
    #: of the image in the units specified by **units**. Possible numeric values are **imageSize**
    #: ≥ (*minWidth*, **minHeight**). The default value is SIZE_ON_SCREEN.Note:The minimum value
    #: of width and height (*minWidth* and **minHeight**) is 10 mm (approximately 0.4 inches).
    imageSize: typing.Union[SymbolicConstant, float] = FIT_TO_PAGE

    #: A SymbolicConstant specifying the units to use for the margins and image size. Possible
    #: values are INCHES and MM. The default value is INCHES.
    units: SymbolicConstant = INCHES

    #: A SymbolicConstant specifying the quality of the image. Possible values are COARSE, MEDIUM,
    #: and FINE. The default value is MEDIUM.
    quality: SymbolicConstant = MEDIUM

    #: A Float specifying the top margin of the paper in the currently selected units (inches or
    #: millimeters). Possible values are **topMargin** ≥ 0. The default value is 0.5.
    topMargin: float = 0.5

    #: A Float specifying the bottom margin of the paper in the currently selected units (inches
    #: or millimeters). Possible values are **bottomMargin** ≥ 0. The default value is 0.5.
    bottomMargin: float = 0.5

    #: A Float specifying the left margin of the paper in the currently selected units (inches or
    #: millimeters). Possible values are **leftMargin** ≥ 0. The default value is 0.5.
    leftMargin: float = 0.5

    #: A Float specifying the right margin of the paper in the currently selected units (inches
    #: or millimeters). Possible values are **rightMargin** ≥ 0. The default value is 0.5.
    rightMargin: float = 0.5

    #: A SymbolicConstant specifying the orientation of the image. Possible values are PORTRAIT
    #: and LANDSCAPE. The default value is PORTRAIT.
    orientation: SymbolicConstant = PORTRAIT

    #: A Boolean specifying whether the output includes the Abaqus logo. The default value is ON.
    logo: Boolean = ON

    #: A Boolean specifying whether the output includes the date. The default value is ON.
    date: Boolean = ON

    @abaqus_method_doc
    def setValues(
        self,
        *,
        imageSize: typing.Union[SymbolicConstant, float] = FIT_TO_PAGE,
        units: SymbolicConstant = INCHES,
        quality: SymbolicConstant = MEDIUM,
        topMargin: float = 0.5,
        bottomMargin: float = 0.5,
        leftMargin: float = 0.5,
        rightMargin: float = 0.5,
        orientation: SymbolicConstant = PORTRAIT,
        logo: Boolean = ON,
        date: Boolean = ON,
    ):
        """This method modifies the PageSetupOptions object.

        Parameters
        ----------
        imageSize
            A SymbolicConstant or a pair of Floats specifying the size of the printed image in the
            currently selected units (inches or millimeters). Possible values are FIT_TO_PAGE and
            SIZE_ON_SCREEN. The default value is FIT_TO_PAGE.Note:The minimum value of width and
            height (*minWidth* and **minHeight**) is 10 mm (approximately 0.4 inches).
        units
            A SymbolicConstant specifying the units to use for the margins and image size. Possible
            values are INCHES and MM. The default value is INCHES.
        quality
            A SymbolicConstant specifying the quality of the image. Possible values are COARSE,
            MEDIUM, and FINE. The default value is MEDIUM.
        topMargin
            A Float specifying the top margin of the paper in the currently selected units (inches
            or millimeters). Possible values are **topMargin** ≥ 0. The default value is 0.5.
        bottomMargin
            A Float specifying the bottom margin of the paper in the currently selected units
            (inches or millimeters). Possible values are **bottomMargin** ≥ 0. The default value is
            0.5.
        leftMargin
            A Float specifying the left margin of the paper in the currently selected units (inches
            or millimeters). Possible values are **leftMargin** ≥ 0. The default value is 0.5.
        rightMargin
            A Float specifying the right margin of the paper in the currently selected units (inches
            or millimeters). Possible values are **rightMargin** ≥ 0. The default value is 0.5.
        orientation
            A SymbolicConstant specifying the orientation of the image. Possible values are PORTRAIT
            and LANDSCAPE. The default value is PORTRAIT.
        logo
            A Boolean specifying whether the output includes the Abaqus logo. The default value is
            ON.
        date
            A Boolean specifying whether the output includes the date. The default value is ON.

        Raises
        ------
        RangeError
            Note: The minimum value of width and height (*minWidth* and **minHeight**) is 10 mm
            (approximately 0.4 inches).
        RangeError: leftMargin and rightMargin must produce image width >= minWidth
            If **leftMargin** + **rightMargin** is out of range.
        RangeError: topMargin and bottomMargin must produce image height >= minHeight
            If **topMargin** + **bottomMargin** is out of range.
        """
        super().setValues(
            imageSize=imageSize,
            units=units,
            quality=quality,
            topMargin=topMargin,
            bottomMargin=bottomMargin,
            leftMargin=leftMargin,
            rightMargin=rightMargin,
            orientation=orientation,
            logo=logo,
            date=date,
        )



import typing

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from .OdbDatumCsys import OdbDatumCsys
from .OdbInstance import OdbInstance
from .OdbMeshElementArray import OdbMeshElementArray
from .OdbMeshNodeArray import OdbMeshNodeArray
from .OdbPretensionSectionArray import OdbPretensionSectionArray
from .OdbRigidBodyArray import OdbRigidBodyArray
from .OdbSet import OdbSet
from .SectionCategory import SectionCategory
from ..Assembly.ConnectorOrientationArray import ConnectorOrientationArray
from ..Property.SectionAssignmentArray import SectionAssignmentArray
from ..Section.Section import Section
from ..UtilityAndView.abaqusConstants import *


@abaqus_class_doc
class OdbAssemblyBase:
    """The OdbAssembly object has no constructor; it is created automatically when an Odb
    object is created. Abaqus creates the **rootAssembly** member when an Odb object is
    created.

    .. note:: 
        This object can be accessed by::

            import odbAccess
            session.odbs[name].rootAssembly
    """

    #: A repository of OdbInstance objects.
    instances: typing.Dict[str, OdbInstance] = {}

    #: A repository of OdbSet objects specifying node sets.
    nodeSets: typing.Dict[str, OdbSet] = {}

    #: A repository of OdbSet objects specifying element sets.
    elementSets: typing.Dict[str, OdbSet] = {}

    #: A repository of OdbSet objects specifying surfaces.
    surfaces: typing.Dict[str, OdbSet] = {}

    #: An :py:class:`~abaqus.Odb.OdbMeshNodeArray.OdbMeshNodeArray` object.
    nodes: OdbMeshNodeArray = []

    #: An :py:class:`~abaqus.Odb.OdbMeshElementArray.OdbMeshElementArray` object.
    elements: OdbMeshElementArray = []

    #: A repository of OdbDatumCsys objects.
    datumCsyses: typing.Dict[str, OdbDatumCsys] = {}

    #: A :py:class:`~abaqus.Property.SectionAssignmentArray.SectionAssignmentArray` object.
    sectionAssignments: SectionAssignmentArray = []

    #: An :py:class:`~abaqus.Odb.OdbRigidBodyArray.OdbRigidBodyArray` object.
    rigidBodies: OdbRigidBodyArray = []

    #: An :py:class:`~abaqus.Odb.OdbPretensionSectionArray.OdbPretensionSectionArray` object.
    pretensionSections: OdbPretensionSectionArray = []

    #: A :py:class:`~abaqus.Assembly.ConnectorOrientationArray.ConnectorOrientationArray` object.
    connectorOrientations: ConnectorOrientationArray = []

    @abaqus_method_doc
    def ConnectorOrientation(
        self,
        region: str,
        localCsys1: OdbDatumCsys = None, 
        axis1: SymbolicConstant = AXIS_1,
        angle1: float = 0,
        orient2sameAs1: Boolean = OFF,
        localCsys2: OdbDatumCsys = None, 
        axis2: SymbolicConstant = AXIS_1,
        angle2: float = 0,
    ):
        """This method assigns a connector orientation to a connector region.

        Parameters
        ----------
        region
            An OdbSet specifying a region.
        localCsys1
            An :py:class:`~abaqus.Odb.OdbDatumCsys.OdbDatumCsys` object specifying the first connector node local coordinate system or
            None, indicating the global coordinate system.
        axis1
            A SymbolicConstant specifying the axis of a cylindrical or spherical datum coordinate
            system about which an additional rotation of the first connector node is applied.
            Possible values are AXIS_1, AXIS_2, and AXIS_3. The default value is AXIS_1.
        angle1
            A Float specifying the angle of the additional rotation about the first connector node
            axis. The default value is 0.0.
        orient2sameAs1
            A Boolean specifying whether the same orientation settings should be used for the second
            node of the connector. The default value is OFF.
        localCsys2
            An :py:class:`~abaqus.Odb.OdbDatumCsys.OdbDatumCsys` object specifying the second connector node local coordinate system or
            None, indicating the global coordinate system.
        axis2
            A SymbolicConstant specifying the axis of a cylindrical or spherical datum coordinate
            system about which an additional rotation of the second connector node is applied.
            Possible values are AXIS_1, AXIS_2, and AXIS_3. The default value is AXIS_1.
        angle2
            A Float specifying the angle of the additional rotation about the second connector node
            axis. The default value is 0.0.

        Raises
        ------
        OdbError: Connector orientation assignment requires element set
            If **region** is not an element set.
        """
        ...

    @abaqus_method_doc
    def SectionAssignment(self, region: str, section: Section):
        """This method is used to assign a section on an assembly or part. Section assignment on
        the assembly is limited to the connector elements only.

        Parameters
        ----------
        region
            An OdbSet specifying a region.
        section
            A :py:class:`~abaqus.Section.Section.Section` object.

        Raises
        ------
        OdbError: Section assignment requires element set
            If **region** is not an element set.
        """
        ...

    @abaqus_method_doc
    def addElements(
        self,
        labels: tuple,
        connectivity: tuple,
        instanceNames: tuple,
        type: str,
        elementSetName: str = "",
        sectionCategory: SectionCategory = None,
    ):
        """This method is used to define elements using nodes defined at the OdbAssembly and/or
        OdbInstance level. For connector elements connected to ground, specify the lone node in
        the connectivity. The position of the ground node cannot be specified. This is a
        limitation.
        Warning:Adding elements not in ascending order of their labels may cause Abaqus/Viewer
        to plot contours incorrectly.

        Parameters
        ----------
        labels
            A sequence of Ints specifying the element labels.
        connectivity
            A sequence of sequences of Ints specifying the nodal connectivity.
        instanceNames
            A sequence of Strings specifying the instanceNames of each node in the nodal
            connectivity array. If the node is defined at the assembly level, the instance name
            should be an empty string
        type
            A String specifying the element type.
        elementSetName
            A String specifying a name for this element set. The default value is the empty string.
        sectionCategory
            A :py:class:`~abaqus.Odb.SectionCategory.SectionCategory` object for this element set.

        Raises
        ------
        OdbError: Addition of this element type is not permitted at the assembly level
            Only certain element types are permitted at the assembly level. e.g., connector
            elements.
        OdbError: Connectivity array must be provided for all element
            If length of label array does not match connectivity data length.
        """
        ...

    @abaqus_method_doc
    def addNodes(self, labels: tuple, coordinates: tuple, nodeSetName: str = None):
        """This method adds nodes to the OdbAssembly object using node labels and coordinates.
        Warning:Adding nodes not in ascending order of their labels may cause Abaqus/Viewer to
        plot contours incorrectly.

        Parameters
        ----------
        labels
            A sequence of Ints specifying the node labels.
        coordinates
            A sequence of sequences of Floats specifying the nodal coordinates.
        nodeSetName
            A String specifying a name for this node set. The default value is None.

        Raises
        ------
        OdbError: Number of node labels and coordinates does not match
            If length of labels does not match length of coordinates.
        OdbError: Node location specification does not correspond to part dimensions
            If width of coordinate array does not match assembly dimension.
        """
        ...

    @abaqus_method_doc
    def RigidBody(
        self,
        referenceNode: str,
        position: str = INPUT,
        isothermal: Boolean = OFF,
        elset: str = "",
        pinNodes: str = "",
        tieNodes: str = "",
        analyticSurface: str = "",
    ):
        """This method defines an OdbRigidBody on the assembly.

        Parameters
        ----------
        referenceNode
            An OdbSet specifying the reference node assigned to the rigid body.
        position
            A symbolic constant specify if the location of the reference node is to be defined by
            the user. Possible values are INPUT and CENTER_OF_MASS. The default value is INPUT.
        isothermal
            A Boolean specifying an isothermal rigid body. The default value is OFF. This parameter
            is used only for a fully coupled thermal stress analysis.
        elset
            An OdbSet specifying an element set assigned to the rigid body.
        pinNodes
            An OdbSet specifying pin-type nodes assigned to the rigid body.
        tieNodes
            An OdbSet specifying tie-type nodes assigned to the rigid body.
        analyticSurface
            An AnalyticSurface specifying the Analytic Rigid Surface assigned to the rigid body.

        Raises
        ------
        OdbError: Rigid body definition requires a node set
            If **referenceNode** is not a node set
        """
        ...



from abqpy.decorators import abaqus_class_doc, abaqus_method_doc


@abaqus_class_doc
class MeanFieldMatrix:
    """The MeanFieldMatrix object specifies the matrix property.

    .. note::
        This object can be accessed by::

            import material
            mdb.models[name].materials[name].constituents[name]
            import odbMaterial
            session.odbs[name].materials[name].constituents[name]

        The corresponding analysis keywords are:

        - CONSTITUENT

    .. versionadded:: 2018
        The `MeanFieldMatrix` class was added.
    """

    @abaqus_method_doc
    def __init__(
        self, name: str, material: str = "", isotropizationCoefficient: float = None
    ):
        """This method creates a MeanFieldMatrix object.

        .. note::
            This function can be accessed by::

                mdb.models[name].materials[name].meanFieldHomogenization.MeanFieldMatrix
                session.odbs[name].materials[name].meanFieldHomogenization.MeanFieldMatrix

        Parameters
        ----------
        name
            A String specifying the constituent repository key.
        material
            A String specifying the name of the material.
        isotropizationCoefficient
            A Float specifying the factor used for scaling the Plastic strain of the constituent
            when calculating the isotropic part of the tangent.

        Returns
        -------
            A MeanFieldMatrix object.

        Raises
        ------
        RangeError
        """
        ...

    @abaqus_method_doc
    def setValues(self):
        """This method modifies the MeanFieldMatrix object.

        Raises
        ------
        RangeError
        """
        ...



import typing

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc
from ..UtilityAndView.abaqusConstants import *


@abaqus_class_doc
class Face:
    """Faces are two-dimensional regions of geometry.

    .. note:: 
        This object can be accessed by::

            import part
            mdb.models[name].parts[name].allInternalSets[name].faces[i]
            mdb.models[name].parts[name].allInternalSurfaces[name].faces[i]
            mdb.models[name].parts[name].allSets[name].faces[i]
            mdb.models[name].parts[name].allSurfaces[name].faces[i]
            mdb.models[name].parts[name].faces[i]
            mdb.models[name].parts[name].sets[name].faces[i]
            mdb.models[name].parts[name].surfaces[name].faces[i]
            import assembly
            mdb.models[name].rootAssembly.allInstances[name].faces[i]
            mdb.models[name].rootAssembly.allInstances[name].sets[name].faces[i]
            mdb.models[name].rootAssembly.allInstances[name].surfaces[name].faces[i]
            mdb.models[name].rootAssembly.allInternalSets[name].faces[i]
            mdb.models[name].rootAssembly.allInternalSurfaces[name].faces[i]
            mdb.models[name].rootAssembly.allSets[name].faces[i]
            mdb.models[name].rootAssembly.allSurfaces[name].faces[i]
            mdb.models[name].rootAssembly.instances[name].faces[i]
            mdb.models[name].rootAssembly.instances[name].sets[name].faces[i]
            mdb.models[name].rootAssembly.instances[name].surfaces[name].faces[i]
            mdb.models[name].rootAssembly.modelInstances[i].sets[name].faces[i]
            mdb.models[name].rootAssembly.modelInstances[i].surfaces[name].faces[i]
            mdb.models[name].rootAssembly.sets[name].faces[i]
            mdb.models[name].rootAssembly.surfaces[name].faces[i]
    """

    #: An Int specifying the index of the face in the FaceArray.
    index: int = None

    #: A Boolean specifying whether the face belongs to the reference representation of the
    #: Part or Instance.
    isReferenceRep: Boolean = OFF

    #: A tuple of tuples of Floats specifying the coordinates. For a face of a shell **pointOn**
    #: specifies the **X**-, **Y**-, and **Z**-coordinates of a point located on the face and the
    #: **X**-, **Y**-, and **Z**-components of the normal to the face.For a face of a solid **pointOn**
    #: specifies the **X**-, **Y**-, and **Z**-coordinates of a point located on the face.
    pointOn: float = None

    #: A tuple of Floats specifying the name of the feature that created this face.
    featureName: float = None

    #: A tuple of Floats specifying the name of the part instance for this face (if
    #: applicable).
    instanceName: float = None

    @abaqus_method_doc
    def getCentroid(self):
        """This method returns the centroid of a face.

        Returns
        -------
        typing.Tuple[float, ...]
            A sequence of Floats specifying the **X**-, **Y**-, and **Z**-coordinates of the centroid of
            the face.
        """
        ...

    @abaqus_method_doc
    def getCurvature(self, point: tuple, uParam: float, vParam: float):
        """This method returns information about the curvature at a location on the face.

        Parameters
        ----------
        point
            A tuple specifying the **X**-, **Y**-, and **Z** coordinates of the point where the curvature
            is desired. If the **point** does not lie on the face it will be projected onto the face.
            This argument and **uParam** and **vParam** are mutually exclusive.
        uParam
            A Float specifying the normalized **U** parameter value at which the curvature is to be
            computed. This value must lie between (0,1). **vParam** must also be specified. This
            argument is mutually exclusive with **point**.
        vParam
            A Float specifying the normalized **V** parameter value at which the curvature is to be
            computed. This value must lie between (0,1).

        Returns
        -------
        dict
            A dictionary with keys 'evaluationPoint', 'principalAxis1', 'principalAxis2',
            'curvature1', 'curvature2' and 'gaussianCurvature'. Where the evaluationPoint specifies
            the location at which the curvature was evaluated. 'principalAxis1' and 'principalAxis2'
            refer to the vectors specifying the two principal axes of the face. 'curvature1' and
            'curvature2' specify the curvatures along the two principal axes.
        """
        ...

    @abaqus_method_doc
    def getElements(self):
        """This method returns an array of element objects that are associated with the face.

        Returns
        -------
        MeshElementArray
            A :py:class:`~abaqus.Mesh.MeshElementArray.MeshElementArray` object which is a sequence of MeshElement objects.

        """
        ...

    @abaqus_method_doc
    def getElementFaces(self, faceSide: SymbolicConstant = BOTH_SIDES):
        """This method returns an array of mesh face objects. Each mesh face object contains the
        element label and the side of the element that lies on the geometric face.

        Parameters
        ----------
        faceSide
            A symbolic constant specifying the side of the geometric face of a solid from which the
            element faces should be retrieved. Possible values are **SIDE1**, **SIDE2**, and
            **BOTH_SIDES**. The default value is **BOTH_SIDES**. For shell faces this option is ignored.

        Returns
        -------
        MeshFaceArray
            A :py:class:`~abaqus.Mesh.MeshFaceArray.MeshFaceArray` object which is a sequence of MeshFace objects.

        """
        ...

    @abaqus_method_doc
    def getNodes(self, faceSide: SymbolicConstant = BOTH_SIDES):
        """This method returns an array of mesh node objects. Each mesh node object contains the
        label of the node that lies on the geometric face.

        Parameters
        ----------
        faceSide
            A symbolic constant specifying the side of the geometric face of a solid from which the
            nodes should be retrieved. Possible values are xo*SIDE1*, **SIDE2**, and **BOTH_SIDES**. The
            default value is **BOTH_SIDES**. For shell faces and for faces with compatible meshes on
            either sides this option is ignored. Otherwise, the nodes on the specified side(s) of
            the face are output.

        Returns
        -------
        MeshNodeArray
            A :py:class:`~abaqus.Mesh.MeshNodeArray.MeshNodeArray` object which is a sequence of MeshNode objects.

        """
        ...

    @abaqus_method_doc
    def getNormal(self, point: tuple = ()):
        """This method returns the normal to a face at the location specified by the **pointOn**
        member. The normal at a different location on the face can be obtained by specifying the
        optional **point** argument.

        Parameters
        ----------
        point
            A tuple specifying the **X**-, **Y**-, and **Z** coordinates of the point where the normal to
            the face is desired. If the **point** does not lie on the face it will be projected onto
            the face.

        Returns
        -------
        typing.Tuple[float, ...]
            A sequence of Floats specifying the **X**-, **Y**-, and **Z**-components of the normal to the
            face.

        Raises
        ------
        An exception is raised if the optional argument **point** is provided but the point cannot
        be projected onto the face.
        """
        ...

    @abaqus_method_doc
    def getSize(self, printResults: str = True):
        """This method returns a Float indicating the area of the face.

        Parameters
        ----------
        printResults
            A Bool specifying whether verbose output is printed. The default value is True.

        Returns
        -------
        float
            A Float.

        """
        ...

    @abaqus_method_doc
    def getEdges(self):
        """This method returns a sequence consisting of the edge ids of the edges on the face.

        Returns
        -------
        typing.Tuple[int, ...]
        A tuple of integers.

        """
        ...

    @abaqus_method_doc
    def getVertices(self):
        """This method returns a sequence consisting of the vertex ids of the vertices of the face.

        Returns
        -------
        typing.Tuple[int, ...]
            A tuple of integers.

        """
        ...

    @abaqus_method_doc
    def getCells(self):
        """This method returns a sequence consisting of the cell ids of the cells to which this
        face belongs.

        Returns
        -------
        typing.Tuple[int, ...]
            A tuple of integers.

        """
        ...

    @abaqus_method_doc
    def getAdjacentFaces(self):
        """This method returns an array of face objects that share at least one edge of the face.

        Returns
        -------
        FaceArray
            A :py:class:`~abaqus.BasicGeometry.FaceArray.FaceArray` object which is a sequence of Face objects.

        """
        ...

    @abaqus_method_doc
    def getFacesByFaceAngle(self, angle: str):
        """This method returns an array of Face objects that are obtained by recursively finding
        adjacent faces that are at an angle of less than or equal to the specified angle.

        Parameters
        ----------
        angle
            A float specifying the value of the face angle.

        Returns
        -------
        FaceArray
            A :py:class:`~abaqus.BasicGeometry.FaceArray.FaceArray` object, which is a sequence of Face objects.

        """
        ...

    @abaqus_method_doc
    def getFacesByCurvature(self):
        """This method returns an array of Face objects that are obtained by recursively finding
        adjacent faces that share the same curvature.

        Returns
        -------
        FaceArray
            A :py:class:`~abaqus.BasicGeometry.FaceArray.FaceArray` object, which is a sequence of Face objects.

        """
        ...

    @abaqus_method_doc
    def isNormalFlipped(self):
        """This method determines whether the normal to the face is flipped from its default
        direction by the use of the flipNormal method on a Part object.

        Returns
        -------
        Boolean
            A Boolean value of True if the normal is flipped and False if not.

        """
        ...

    @abaqus_method_doc
    def getCADAttributes(self):
        """This method returns an array of CAD attribute strings associated with the Face when the
        part was created from CAD data.

        Returns
        -------
        typing.List[str]
            An array of String.

        """
        ...
